@@ -19,18 +19,24 @@ import os
 import platform
 
 from parsl.version import VERSION
-from parsl.app.app import bash_app, python_app
+from parsl.app.app import bash_app, join_app, python_app
+from parsl.config import Config
 from parsl.executors import ThreadPoolExecutor
-from parsl.executors import IPyParallelExecutor
 from parsl.executors import HighThroughputExecutor
 from parsl.executors import ExtremeScaleExecutor
+from parsl.executors import WorkQueueExecutor
 from parsl.log_utils import set_stream_logger
 from parsl.log_utils import set_file_logger
+from parsl.monitoring import MonitoringHub
 
 from parsl.data_provider.files import File
 
 from parsl.dataflow.dflow import DataFlowKernel, DataFlowKernelLoader
 
+import multiprocessing
+if platform.system() == 'Darwin':
+    multiprocessing.set_start_method('fork', force=True)
+
 __author__ = 'The Parsl Team'
 __version__ = VERSION
 
@@ -41,8 +47,10 @@ __all__ = [
     # decorators
     'bash_app',
     'python_app',
+    'join_app',
 
     # core
+    'Config',
     'DataFlowKernel',
     'File',
 
@@ -55,7 +63,10 @@ __all__ = [
     'ThreadPoolExecutor',
     'HighThroughputExecutor',
     'ExtremeScaleExecutor',
-    'IPyParallelExecutor',
+    'WorkQueueExecutor',
+
+    # monitoring
+    'MonitoringHub',
 ]
 
 clear = DataFlowKernelLoader.clear
@@ -64,14 +75,7 @@ dfk = DataFlowKernelLoader.dfk
 wait_for_current_tasks = DataFlowKernelLoader.wait_for_current_tasks
 
 
-class NullHandler(logging.Handler):
-    """Setup default logging to /dev/null since this is library."""
-
-    def emit(self, record):
-        pass
-
-
-logging.getLogger('parsl').addHandler(NullHandler())
+logging.getLogger('parsl').addHandler(logging.NullHandler())
 
 if platform.system() == 'Darwin':
     os.environ['OBJC_DISABLE_INITIALIZE_FORK_SAFETY'] = 'YES'
