@@ -35,7 +35,7 @@ def timeout(f, seconds):
 class PythonApp(AppBase):
     """Extends AppBase to cover the Python App."""
 
-    def __init__(self, func, data_flow_kernel=None, cache=False, executors='all', ignore_for_cache=[]):
+    def __init__(self, func, data_flow_kernel=None, cache=False, executors='all', ignore_for_cache=[], join=False):
         super().__init__(
             wrap_error(func),
             data_flow_kernel=data_flow_kernel,
@@ -43,6 +43,7 @@ class PythonApp(AppBase):
             cache=cache,
             ignore_for_cache=ignore_for_cache
         )
+        self.join = join
 
     def __call__(self, *args, **kwargs):
         """This is where the call to a python app is handled.
@@ -73,9 +74,9 @@ class PythonApp(AppBase):
 
         app_fut = dfk.submit(func, app_args=args,
                              executors=self.executors,
-                             fn_hash=self.func_hash,
                              cache=self.cache,
                              ignore_for_cache=self.ignore_for_cache,
-                             app_kwargs=kwargs)
+                             app_kwargs=invocation_kwargs,
+                             join=self.join)
 
         return app_fut
