@@ -1,6 +1,5 @@
 from parsl.config import Config
 from parsl.executors import HighThroughputExecutor
-from parsl.addresses import address_by_hostname
 from parsl.launchers import MpiRunLauncher
 from parsl.providers import CobaltProvider
 
@@ -11,7 +10,6 @@ config = Config(
             label="cooley_htex",
             worker_debug=False,
             cores_per_worker=1,
-            address=address_by_hostname(),
             provider=CobaltProvider(
                 queue='debug',
                 account='YOUR_ACCOUNT',  # project name to submit the job
