@@ -1,8 +1,10 @@
 import logging
-import sys
 import threading
 import time
 
+from typing import Sequence
+
+from parsl.executors.base import ParslExecutor
 from parsl.dataflow.task_status_poller import TaskStatusPoller
 
 logger = logging.getLogger(__name__)
@@ -30,7 +32,7 @@ class FlowControl(object):
             callback()
 
     This logic ensures that the callbacks are activated with a maximum delay
-    of `interval` for systems with infrequent events as well as systems which would
+    of ``interval`` for systems with infrequent events as well as systems which would
     generate large bursts of events.
 
     Once a callback is triggered, the callback generally runs a strategy
@@ -54,7 +56,6 @@ class FlowControl(object):
              - threshold (int) : Tasks after which the callback is triggered
              - interval (int) : seconds after which timer expires
         """
-        self.dfk = dfk
         self.threshold = threshold
         self.interval = interval
         self.cb_args = args
@@ -69,7 +70,7 @@ class FlowControl(object):
         self._thread.daemon = True
         self._thread.start()
 
-    def _wake_up_timer(self, kill_event):
+    def _wake_up_timer(self, kill_event: threading.Event) -> None:
         """Internal. This is the function that the thread will execute.
         waits on an event so that the thread can make a quick exit when close() is called
 
@@ -87,37 +88,22 @@ class FlowControl(object):
             if time_to_die:
                 return
 
-            if prev == self._wake_up_time:
-                self.make_callback(kind='timer')
-            else:
-                print("Sleeping a bit more")
+            self.make_callback()
 
-    def notify(self, event_id):
-        """Let the FlowControl system know that there is an event."""
-        self._event_buffer.extend([event_id])
-        self._event_count += 1
-        if self._event_count >= self.threshold:
-            logger.debug("Eventcount >= threshold")
-            self.make_callback(kind="event")
-
-    def make_callback(self, kind=None):
+    def make_callback(self) -> None:
         """Makes the callback and resets the timer.
-
-        KWargs:
-               - kind (str): Default=None, used to pass information on what
-                 triggered the callback
         """
         self._wake_up_time = time.time() + self.interval
         try:
-            self.callback(tasks=self._event_buffer, kind=kind)
+            self.callback(tasks=self._event_buffer)
         except Exception:
             logger.error("Flow control callback threw an exception - logging and proceeding anyway", exc_info=True)
         self._event_buffer = []
 
-    def add_executors(self, executors):
+    def add_executors(self, executors: Sequence[ParslExecutor]) -> None:
         self.task_status_poller.add_executors(executors)
 
-    def close(self):
+    def close(self) -> None:
         """Merge the threads and terminate."""
         self._kill_event.set()
         self._thread.join()
@@ -170,7 +156,7 @@ class Timer(object):
         self._thread.daemon = True
         self._thread.start()
 
-    def _wake_up_timer(self, kill_event):
+    def _wake_up_timer(self, kill_event: threading.Event) -> None:
         """Internal. This is the function that the thread will execute.
         waits on an event so that the thread can make a quick exit when close() is called
 
@@ -190,52 +176,18 @@ class Timer(object):
                 return
 
             if prev == self._wake_up_time:
-                self.make_callback(kind='timer')
+                self.make_callback()
             else:
                 print("Sleeping a bit more")
 
-    def make_callback(self, kind=None):
+    def make_callback(self) -> None:
         """Makes the callback and resets the timer.
         """
         self._wake_up_time = time.time() + self.interval
         self.callback(*self.cb_args)
 
-    def close(self):
+    def close(self) -> None:
         """Merge the threads and terminate.
         """
         self._kill_event.set()
         self._thread.join()
-
-
-if __name__ == "__main__":
-
-    def foo():
-        print("Callback made at :", time.time())
-
-    timer = Timer(foo)
-
-    time.sleep(60)
-    timer.close()
-    exit(0)
-
-    print("This is broken")
-
-    def cback(*args):
-        print("*" * 40)
-        print("Callback at {0} with args : {1}".format(time.time(), args))
-        print("*" * 40)
-
-    fc = FlowControl(cback)
-
-    print("Testing")
-    print("Press E(Enter) to create and event, X(Enter) to exit")
-    while True:
-        x = sys.stdin.read(1)
-        if x.lower() == 'e':
-            print("Event")
-            fc.notify()
-        elif x.lower() == 'x':
-            print("Exiting ...")
-            break
-        else:
-            print("Continuing.. got[%s]", x)
