@@ -3,14 +3,14 @@ import typeguard
 import logging
 import threading
 import queue
+import datetime
 import pickle
-from multiprocessing import Process, Queue
-from typing import Dict, List, Optional, Tuple, Union
+from multiprocessing import Queue
+from typing import Dict, Sequence  # noqa F401 (used in type annotation)
+from typing import List, Optional, Tuple, Union
 import math
 
-from ipyparallel.serialize import pack_apply_message
-from ipyparallel.serialize import deserialize_object
-
+from parsl.serialize import pack_apply_message, deserialize
 from parsl.app.errors import RemoteExceptionWrapper
 from parsl.executors.high_throughput import zmq_pipes
 from parsl.executors.high_throughput import interchange
@@ -20,21 +20,20 @@ from parsl.executors.errors import (
     UnsupportedFeatureError
 )
 
-from parsl.executors.status_handling import StatusHandlingExecutor
+from parsl.executors.status_handling import BlockProviderExecutor
 from parsl.providers.provider_base import ExecutionProvider
 from parsl.data_provider.staging import Staging
 from parsl.addresses import get_all_addresses
+from parsl.process_loggers import wrap_with_logs
 
+from parsl.multiprocessing import ForkProcess
 from parsl.utils import RepresentationMixin
 from parsl.providers import LocalProvider
 
 logger = logging.getLogger(__name__)
 
-BUFFER_THRESHOLD = 1024 * 1024
-ITEM_THRESHOLD = 1024
 
-
-class HighThroughputExecutor(StatusHandlingExecutor, RepresentationMixin):
+class HighThroughputExecutor(BlockProviderExecutor, RepresentationMixin):
     """Executor designed for cluster-scale
 
     The HighThroughputExecutor system has the following components:
@@ -79,7 +78,6 @@ class HighThroughputExecutor(StatusHandlingExecutor, RepresentationMixin):
         :class:`~parsl.providers.condor.condor.Condor`,
         :class:`~parsl.providers.googlecloud.googlecloud.GoogleCloud`,
         :class:`~parsl.providers.gridEngine.gridEngine.GridEngine`,
-        :class:`~parsl.providers.jetstream.jetstream.Jetstream`,
         :class:`~parsl.providers.local.local.Local`,
         :class:`~parsl.providers.sge.sge.GridEngine`,
         :class:`~parsl.providers.slurm.slurm.Slurm`, or
@@ -96,7 +94,7 @@ class HighThroughputExecutor(StatusHandlingExecutor, RepresentationMixin):
 
     address : string
         An address to connect to the main Parsl process which is reachable from the network in which
-        workers will be running. This can be either a hostname as returned by `hostname` or an
+        workers will be running. This can be either a hostname as returned by ``hostname`` or an
         IP address. Most login nodes on clusters have several network interfaces available, only
         some of which can be reached from the compute nodes.
         By default, the executor will attempt to enumerate and connect through all possible addresses.
@@ -132,7 +130,23 @@ class HighThroughputExecutor(StatusHandlingExecutor, RepresentationMixin):
         the there's sufficient memory for each worker. Default: None
 
     max_workers : int
-        Caps the number of workers launched by the manager. Default: infinity
+        Caps the number of workers launched per node. Default: infinity
+
+    cpu_affinity: string
+        Whether or how each worker process sets thread affinity. Options are "none" to forgo
+        any CPU affinity configuration, "block" to assign adjacent cores to workers
+        (ex: assign 0-1 to worker 0, 2-3 to worker 1), and
+        "alternating" to assign cores to workers in round-robin
+        (ex: assign 0,2 to worker 0, 1,3 to worker 1).
+
+    available_accelerators: int | list
+        Accelerators available for workers to use. Each worker will be pinned to exactly one of the provided
+        accelerators, and no more workers will be launched than the number of accelerators.
+
+        Either provide the list of accelerator names or the number available. If a number is provided,
+        Parsl will create names as integers starting with 0.
+
+        default: empty list
 
     prefetch_capacity : int
         Number of tasks that could be prefetched over available worker capacity.
@@ -175,24 +189,26 @@ class HighThroughputExecutor(StatusHandlingExecutor, RepresentationMixin):
                  cores_per_worker: float = 1.0,
                  mem_per_worker: Optional[float] = None,
                  max_workers: Union[int, float] = float('inf'),
+                 cpu_affinity: str = 'none',
+                 available_accelerators: Union[int, Sequence[str]] = (),
                  prefetch_capacity: int = 0,
                  heartbeat_threshold: int = 120,
                  heartbeat_period: int = 30,
                  poll_period: int = 10,
                  address_probe_timeout: Optional[int] = None,
                  managed: bool = True,
-                 worker_logdir_root: Optional[str] = None):
+                 worker_logdir_root: Optional[str] = None,
+                 block_error_handler: bool = True):
 
         logger.debug("Initializing HighThroughputExecutor")
 
-        StatusHandlingExecutor.__init__(self, provider)
+        BlockProviderExecutor.__init__(self, provider=provider, block_error_handler=block_error_handler)
         self.label = label
         self.launch_cmd = launch_cmd
         self.worker_debug = worker_debug
         self.storage_access = storage_access
         self.working_dir = working_dir
         self.managed = managed
-        self.blocks = {}  # type: Dict[str, str]
         self.cores_per_worker = cores_per_worker
         self.mem_per_worker = mem_per_worker
         self.max_workers = max_workers
@@ -215,11 +231,21 @@ class HighThroughputExecutor(StatusHandlingExecutor, RepresentationMixin):
                 self.provider.cores_per_node is not None:
             cpu_slots = math.floor(self.provider.cores_per_node / cores_per_worker)
 
-        self.workers_per_node = min(max_workers, mem_slots, cpu_slots)
-        if self.workers_per_node == float('inf'):
-            self.workers_per_node = 1  # our best guess-- we do not have any provider hints
+        # Set the list of available accelerators
+        if isinstance(available_accelerators, int):
+            # If the user provide an integer, create some names for them
+            available_accelerators = list(map(str, range(available_accelerators)))
+        self.available_accelerators = list(available_accelerators)
+
+        # Determine the number of workers per node
+        self._workers_per_node = min(max_workers, mem_slots, cpu_slots)
+        if len(self.available_accelerators) > 0:
+            self._workers_per_node = min(self._workers_per_node, len(available_accelerators))
+        if self._workers_per_node == float('inf'):
+            self._workers_per_node = 1  # our best guess-- we do not have any provider hints
 
         self._task_counter = 0
+        self.run_id = None  # set to the correct run_id in dfk
         self.hub_address = None  # set to the correct hub address in dfk
         self.hub_port = None  # set to the correct hub port in dfk
         self.worker_ports = worker_ports
@@ -230,6 +256,7 @@ class HighThroughputExecutor(StatusHandlingExecutor, RepresentationMixin):
         self.poll_period = poll_period
         self.run_dir = '.'
         self.worker_logdir_root = worker_logdir_root
+        self.cpu_affinity = cpu_affinity
 
         if not launch_cmd:
             self.launch_cmd = ("process_worker_pool.py {debug} {max_workers} "
@@ -244,7 +271,11 @@ class HighThroughputExecutor(StatusHandlingExecutor, RepresentationMixin):
                                "--block_id={{block_id}} "
                                "--hb_period={heartbeat_period} "
                                "{address_probe_timeout_string} "
-                               "--hb_threshold={heartbeat_threshold} ")
+                               "--hb_threshold={heartbeat_threshold} "
+                               "--cpu-affinity {cpu_affinity} "
+                               "--available-accelerators {accelerators}")
+
+    radio_mode = "htex"
 
     def initialize_scaling(self):
         """ Compose the launch command and call the scale_out
@@ -275,18 +306,24 @@ class HighThroughputExecutor(StatusHandlingExecutor, RepresentationMixin):
                                        heartbeat_period=self.heartbeat_period,
                                        heartbeat_threshold=self.heartbeat_threshold,
                                        poll_period=self.poll_period,
-                                       logdir=worker_logdir)
+                                       logdir=worker_logdir,
+                                       cpu_affinity=self.cpu_affinity,
+                                       accelerators=" ".join(self.available_accelerators))
         self.launch_cmd = l_cmd
         logger.debug("Launch command: {}".format(self.launch_cmd))
 
         self._scaling_enabled = True
         logger.debug("Starting HighThroughputExecutor with provider:\n%s", self.provider)
+
+        # TODO: why is this a provider property?
+        block_ids = []
         if hasattr(self.provider, 'init_blocks'):
             try:
-                self.scale_out(blocks=self.provider.init_blocks)
+                block_ids = self.scale_out(blocks=self.provider.init_blocks)
             except Exception as e:
                 logger.error("Scaling out failed: {}".format(e))
                 raise e
+        return block_ids
 
     def start(self):
         """Create the Interchange process and connect to it.
@@ -299,12 +336,14 @@ class HighThroughputExecutor(StatusHandlingExecutor, RepresentationMixin):
 
         self._queue_management_thread = None
         self._start_queue_management_thread()
-        self._start_local_queue_process()
+        self._start_local_interchange_process()
 
         logger.debug("Created management thread: {}".format(self._queue_management_thread))
 
-        self.initialize_scaling()
+        block_ids = self.initialize_scaling()
+        return block_ids
 
+    @wrap_with_logs
     def _queue_management_worker(self):
         """Listen to the queue for task status messages and handle them.
 
@@ -324,18 +363,6 @@ class HighThroughputExecutor(StatusHandlingExecutor, RepresentationMixin):
                "exception" : serialized exception object, on failure
             }
 
-        We do not support these yet, but they could be added easily.
-
-        .. code:: python
-
-            {
-               "task_id" : <task_id>
-               "cpu_stat" : <>
-               "mem_stat" : <>
-               "io_stat"  : <>
-               "started"  : tstamp
-            }
-
         The `None` message is a die request.
         """
         logger.debug("[MTHREAD] queue management worker starting")
@@ -367,80 +394,80 @@ class HighThroughputExecutor(StatusHandlingExecutor, RepresentationMixin):
                     for serialized_msg in msgs:
                         try:
                             msg = pickle.loads(serialized_msg)
-                            tid = msg['task_id']
                         except pickle.UnpicklingError:
                             raise BadMessage("Message received could not be unpickled")
 
-                        except Exception:
-                            raise BadMessage("Message received does not contain 'task_id' field")
-
-                        if tid == -1 and 'exception' in msg:
-                            logger.warning("Executor shutting down due to exception from interchange")
-                            exception, _ = deserialize_object(msg['exception'])
-                            self.set_bad_state_and_fail_all(exception)
-                            break
-
-                        task_fut = self.tasks[tid]
-
-                        if 'result' in msg:
-                            result, _ = deserialize_object(msg['result'])
-                            task_fut.set_result(result)
-
-                        elif 'exception' in msg:
+                        if msg['type'] == 'heartbeat':
+                            continue
+                        elif msg['type'] == 'result':
                             try:
-                                s, _ = deserialize_object(msg['exception'])
-                                # s should be a RemoteExceptionWrapper... so we can reraise it
-                                if isinstance(s, RemoteExceptionWrapper):
-                                    try:
-                                        s.reraise()
-                                    except Exception as e:
-                                        task_fut.set_exception(e)
-                                elif isinstance(s, Exception):
-                                    task_fut.set_exception(s)
-                                else:
-                                    raise ValueError("Unknown exception-like type received: {}".format(type(s)))
-                            except Exception as e:
-                                # TODO could be a proper wrapped exception?
-                                task_fut.set_exception(
-                                    DeserializationError("Received exception, but handling also threw an exception: {}".format(e)))
+                                tid = msg['task_id']
+                            except Exception:
+                                raise BadMessage("Message received does not contain 'task_id' field")
+
+                            if tid == -1 and 'exception' in msg:
+                                logger.warning("Executor shutting down due to exception from interchange")
+                                exception = deserialize(msg['exception'])
+                                self.set_bad_state_and_fail_all(exception)
+                                break
+
+                            task_fut = self.tasks.pop(tid)
+
+                            if 'result' in msg:
+                                result = deserialize(msg['result'])
+                                task_fut.set_result(result)
+
+                            elif 'exception' in msg:
+                                try:
+                                    s = deserialize(msg['exception'])
+                                    # s should be a RemoteExceptionWrapper... so we can reraise it
+                                    if isinstance(s, RemoteExceptionWrapper):
+                                        try:
+                                            s.reraise()
+                                        except Exception as e:
+                                            task_fut.set_exception(e)
+                                    elif isinstance(s, Exception):
+                                        task_fut.set_exception(s)
+                                    else:
+                                        raise ValueError("Unknown exception-like type received: {}".format(type(s)))
+                                except Exception as e:
+                                    # TODO could be a proper wrapped exception?
+                                    task_fut.set_exception(
+                                        DeserializationError("Received exception, but handling also threw an exception: {}".format(e)))
+                            else:
+                                raise BadMessage("Message received is neither result or exception")
                         else:
-                            raise BadMessage("Message received is neither result or exception")
+                            raise BadMessage("Message received with unknown type {}".format(msg['type']))
 
             if not self.is_alive:
                 break
         logger.info("[MTHREAD] queue management worker finished")
 
-    # When the executor gets lost, the weakref callback will wake up
-    # the queue management thread.
-    def weakref_cb(self, q=None):
-        """We do not use this yet."""
-        q.put(None)
-
-    def _start_local_queue_process(self):
+    def _start_local_interchange_process(self):
         """ Starts the interchange process locally
 
         Starts the interchange process locally and uses an internal command queue to
         get the worker task and result ports that the interchange has bound to.
         """
         comm_q = Queue(maxsize=10)
-        self.queue_proc = Process(target=interchange.starter,
-                                  args=(comm_q,),
-                                  kwargs={"client_ports": (self.outgoing_q.port,
-                                                           self.incoming_q.port,
-                                                           self.command_client.port),
-                                          "worker_ports": self.worker_ports,
-                                          "worker_port_range": self.worker_port_range,
-                                          "hub_address": self.hub_address,
-                                          "hub_port": self.hub_port,
-                                          "logdir": "{}/{}".format(self.run_dir, self.label),
-                                          "heartbeat_threshold": self.heartbeat_threshold,
-                                          "poll_period": self.poll_period,
-                                          "logging_level": logging.DEBUG if self.worker_debug else logging.INFO
-                                  },
-                                  daemon=True,
-                                  name="HTEX-Interchange"
+        self.interchange_proc = ForkProcess(target=interchange.starter,
+                                            args=(comm_q,),
+                                            kwargs={"client_ports": (self.outgoing_q.port,
+                                                                     self.incoming_q.port,
+                                                                     self.command_client.port),
+                                                    "worker_ports": self.worker_ports,
+                                                    "worker_port_range": self.worker_port_range,
+                                                    "hub_address": self.hub_address,
+                                                    "hub_port": self.hub_port,
+                                                    "logdir": "{}/{}".format(self.run_dir, self.label),
+                                                    "heartbeat_threshold": self.heartbeat_threshold,
+                                                    "poll_period": self.poll_period,
+                                                    "logging_level": logging.DEBUG if self.worker_debug else logging.INFO
+                                            },
+                                            daemon=True,
+                                            name="HTEX-Interchange"
         )
-        self.queue_proc.start()
+        self.interchange_proc.start()
         try:
             (self.worker_task_port, self.worker_result_port) = comm_q.get(block=True, timeout=120)
         except queue.Empty:
@@ -461,7 +488,7 @@ class HighThroughputExecutor(StatusHandlingExecutor, RepresentationMixin):
             logger.debug("Started queue management thread")
 
         else:
-            logger.debug("Management thread already exists, returning")
+            logger.error("Management thread already exists, returning")
 
     def hold_worker(self, worker_id):
         """Puts a worker on hold, preventing scheduling of additional tasks to it.
@@ -476,7 +503,7 @@ class HighThroughputExecutor(StatusHandlingExecutor, RepresentationMixin):
             Worker id to be put on hold
         """
         c = self.command_client.run("HOLD_WORKER;{}".format(worker_id))
-        logger.debug("Sent hold request to worker: {}".format(worker_id))
+        logger.debug("Sent hold request to manager: {}".format(worker_id))
         return c
 
     @property
@@ -519,10 +546,10 @@ class HighThroughputExecutor(StatusHandlingExecutor, RepresentationMixin):
 
         Args:
             - func (callable) : Callable function
-            - *args (list) : List of arbitrary positional arguments.
+            - args (list) : List of arbitrary positional arguments.
 
         Kwargs:
-            - **kwargs (dict) : A dictionary of arbitrary keyword args for func.
+            - kwargs (dict) : A dictionary of arbitrary keyword args for func.
 
         Returns:
               Future
@@ -545,12 +572,13 @@ class HighThroughputExecutor(StatusHandlingExecutor, RepresentationMixin):
             args_to_print = tuple([arg if len(repr(arg)) < 100 else (repr(arg)[:100] + '...') for arg in args])
         logger.debug("Pushing function {} to queue with args {}".format(func, args_to_print))
 
-        self.tasks[task_id] = Future()
+        fut = Future()
+        fut.parsl_executor_task_id = task_id
+        self.tasks[task_id] = fut
 
         try:
             fn_buf = pack_apply_message(func, args, kwargs,
-                                        buffer_threshold=1024 * 1024,
-                                        item_threshold=1024)
+                                        buffer_threshold=1024 * 1024)
         except TypeError:
             raise SerializationError(func.__name__)
 
@@ -561,32 +589,33 @@ class HighThroughputExecutor(StatusHandlingExecutor, RepresentationMixin):
         self.outgoing_q.put(msg)
 
         # Return the future
-        return self.tasks[task_id]
+        return fut
 
     @property
     def scaling_enabled(self):
         return self._scaling_enabled
 
-    def scale_out(self, blocks=1):
-        """Scales out the number of blocks by "blocks"
+    def create_monitoring_info(self, status):
+        """ Create a msg for monitoring based on the poll status
 
-        Raises:
-             NotImplementedError
         """
-        r = []
-        for i in range(blocks):
-            external_block_id = str(len(self.blocks))
-            launch_cmd = self.launch_cmd.format(block_id=external_block_id)
-            internal_block = self.provider.submit(launch_cmd, 1)
-            logger.debug("Launched block {}->{}".format(external_block_id, internal_block))
-            if not internal_block:
-                raise(ScalingFailed(self.provider.label,
-                                    "Attempts to provision nodes via provider has failed"))
-            r.extend([external_block_id])
-            self.blocks[external_block_id] = internal_block
-        return r
+        msg = []
+        for bid, s in status.items():
+            d = {}
+            d['run_id'] = self.run_id
+            d['status'] = s.status_name
+            d['timestamp'] = datetime.datetime.now()
+            d['executor_label'] = self.label
+            d['job_id'] = self.blocks.get(bid, None)
+            d['block_id'] = bid
+            msg.append(d)
+        return msg
 
-    def scale_in(self, blocks=None, block_ids=[]):
+    @property
+    def workers_per_node(self) -> Union[int, float]:
+        return self._workers_per_node
+
+    def scale_in(self, blocks=None, block_ids=[], force=True, max_idletime=None):
         """Scale in the number of active blocks by specified amount.
 
         The scale in method here is very rude. It doesn't give the workers
@@ -599,47 +628,85 @@ class HighThroughputExecutor(StatusHandlingExecutor, RepresentationMixin):
         blocks : int
              Number of blocks to terminate and scale_in by
 
+        force : Bool
+             Used along with blocks to indicate whether blocks should be terminated by force.
+             When force = True, we will kill blocks regardless of the blocks being busy
+             When force = False, Only idle blocks will be terminated.
+             If the # of ``idle_blocks`` < ``blocks``, the list of jobs marked for termination
+             will be in the range: 0 - ``blocks``.
+
+        max_idletime: float
+             A time to indicate how long a block can be idle.
+             Used along with force = False to kill blocks that have been idle for that long.
+
         block_ids : list
              List of specific block ids to terminate. Optional
 
-        Raises:
-             NotImplementedError
+        Returns
+        -------
+        List of job_ids marked for termination
         """
-
+        logger.debug(f"Scale in called, blocks={blocks}, block_ids={block_ids}")
         if block_ids:
             block_ids_to_kill = block_ids
         else:
-            block_ids_to_kill = list(self.blocks.keys())[:blocks]
+            managers = self.connected_managers
+            block_info = {}  # block id -> list( tasks, idle duration )
+            for manager in managers:
+                if not manager['active']:
+                    continue
+                b_id = manager['block_id']
+                if b_id not in block_info:
+                    block_info[b_id] = [0, float('inf')]
+                block_info[b_id][0] += manager['tasks']
+                block_info[b_id][1] = min(block_info[b_id][1], manager['idle_duration'])
+
+            sorted_blocks = sorted(block_info.items(), key=lambda item: (item[1][1], item[1][0]))
+            logger.debug(f"Scale in selecting from {len(sorted_blocks)} blocks")
+            if force is True:
+                block_ids_to_kill = [x[0] for x in sorted_blocks[:blocks]]
+            else:
+                if not max_idletime:
+                    block_ids_to_kill = [x[0] for x in sorted_blocks if x[1][0] == 0][:blocks]
+                else:
+                    block_ids_to_kill = []
+                    for x in sorted_blocks:
+                        if x[1][1] > max_idletime and x[1][0] == 0:
+                            block_ids_to_kill.append(x[0])
+                            if len(block_ids_to_kill) == blocks:
+                                break
+                logger.debug("Selected idle block ids to kill: {}".format(
+                    block_ids_to_kill))
+                if len(block_ids_to_kill) < blocks:
+                    logger.warning(f"Could not find enough blocks to kill: wanted {blocks} but only selected {len(block_ids_to_kill)}")
 
         # Hold the block
         for block_id in block_ids_to_kill:
             self._hold_block(block_id)
 
         # Now kill via provider
-        to_kill = [self.blocks.pop(bid) for bid in block_ids_to_kill]
+        # Potential issue with multiple threads trying to remove the same blocks
+        to_kill = [self.blocks[bid] for bid in block_ids_to_kill if bid in self.blocks]
 
         r = self.provider.cancel(to_kill)
+        job_ids = self._filter_scale_in_ids(to_kill, r)
 
-        return self._filter_scale_in_ids(to_kill, r)
+        # to_kill block_ids are fetched from self.blocks
+        # If a block_id is in self.block, it must exist in self.block_mapping
+        block_ids_killed = [self.block_mapping[jid] for jid in job_ids]
 
-    def _get_job_ids(self) -> List[object]:
-        return list(self.blocks.values())
+        return block_ids_killed
 
-    def shutdown(self, hub=True, targets='all', block=False):
+    def _get_launch_command(self, block_id: str) -> str:
+        if self.launch_cmd is None:
+            raise ScalingFailed(self, "No launch command")
+        launch_cmd = self.launch_cmd.format(block_id=block_id)
+        return launch_cmd
+
+    def shutdown(self):
         """Shutdown the executor, including all workers and controllers.
-
-        This is not implemented.
-
-        Kwargs:
-            - hub (Bool): Whether the hub should be shutdown, Default: True,
-            - targets (list of ints| 'all'): List of block id's to kill, Default: 'all'
-            - block (Bool): To block for confirmations or not
-
-        Raises:
-             NotImplementedError
         """
 
         logger.info("Attempting HighThroughputExecutor shutdown")
-        self.queue_proc.terminate()
+        self.interchange_proc.terminate()
         logger.info("Finished HighThroughputExecutor shutdown attempt")
-        return True
