@@ -13,31 +13,23 @@ import queue
 import threading
 import json
 
+from typing import cast, Any, Dict, Set
+
+from parsl.utils import setproctitle
 from parsl.version import VERSION as PARSL_VERSION
-from ipyparallel.serialize import serialize_object
+from parsl.serialize import ParslSerializer
+serialize_object = ParslSerializer().serialize
 
 from parsl.app.errors import RemoteExceptionWrapper
+from parsl.executors.high_throughput.manager_record import ManagerRecord
 from parsl.monitoring.message_type import MessageType
+from parsl.process_loggers import wrap_with_logs
 
 
-LOOP_SLOWDOWN = 0.0  # in seconds
 HEARTBEAT_CODE = (2 ** 32) - 1
 PKL_HEARTBEAT_CODE = pickle.dumps((2 ** 32) - 1)
 
 
-class ShutdownRequest(Exception):
-    ''' Exception raised when any async component receives a ShutdownRequest
-    '''
-    def __init__(self):
-        self.tstamp = time.time()
-
-    def __repr__(self):
-        return "Shutdown request received at {}".format(self.tstamp)
-
-    def __str__(self):
-        return self.__repr__()
-
-
 class ManagerLost(Exception):
     ''' Task lost due to manager loss. Manager is considered lost when multiple heartbeats
     have been missed.
@@ -54,23 +46,6 @@ class ManagerLost(Exception):
         return self.__repr__()
 
 
-class BadRegistration(Exception):
-    ''' A new Manager tried to join the executor with a BadRegistration message
-    '''
-    def __init__(self, worker_id, critical=False):
-        self.worker_id = worker_id
-        self.tstamp = time.time()
-        self.handled = "critical" if critical else "suppressed"
-
-    def __repr__(self):
-        return "Manager {} attempted to register with a bad registration message. Caused a {} failure".format(
-            self.worker_id,
-            self.handled)
-
-    def __str__(self):
-        return self.__repr__()
-
-
 class VersionMismatch(Exception):
     ''' Manager and Interchange versions do not match
     '''
@@ -80,8 +55,8 @@ class VersionMismatch(Exception):
 
     def __repr__(self):
         return "Manager version info {} does not match interchange version info {}, causing a critical failure".format(
-            self.interchange_version,
-            self.manager_version)
+            self.manager_version,
+            self.interchange_version)
 
     def __str__(self):
         return self.__repr__()
@@ -94,24 +69,22 @@ class Interchange(object):
     2. Allow for workers to join and leave the union
     3. Detect workers that have failed using heartbeats
     4. Service single and batch requests from workers
-    5. Be aware of requests worker resource capacity,
-       eg. schedule only jobs that fit into walltime.
 
     TODO: We most likely need a PUB channel to send out global commands, like shutdown
     """
     def __init__(self,
                  client_address="127.0.0.1",
                  interchange_address="127.0.0.1",
-                 hub_address=None,
                  client_ports=(50055, 50056, 50057),
                  worker_ports=None,
                  worker_port_range=(54000, 55000),
+                 hub_address=None,
                  hub_port=None,
                  heartbeat_threshold=60,
                  logdir=".",
                  logging_level=logging.INFO,
                  poll_period=10,
-             ):
+             ) -> None:
         """
         Parameters
         ----------
@@ -121,10 +94,6 @@ class Interchange(object):
         interchange_address : str
              The ip address at which the workers will be able to reach the Interchange. Default: "127.0.0.1"
 
-        hub_address : str
-             The ip address at which the interchange can send info about managers to when monitoring is enabled.
-             This is passed via dfk and executor automatically. Default: None (meaning monitoring disabled)
-
         client_ports : triple(int, int, int)
              The ports at which the client can be reached
 
@@ -133,7 +102,11 @@ class Interchange(object):
 
         worker_port_range : tuple(int, int)
              The interchange picks ports at random from the range which will be used by workers.
-             This is overridden when the worker_ports option is set. Defauls: (54000, 55000)
+             This is overridden when the worker_ports option is set. Default: (54000, 55000)
+
+        hub_address : str
+             The ip address at which the interchange can send info about managers to when monitoring is enabled.
+             This is passed via dfk and executor automatically. Default: None (meaning monitoring disabled)
 
         hub_port : str
              The port at which the interchange can send info about managers to when monitoring is enabled.
@@ -146,7 +119,7 @@ class Interchange(object):
              Parsl log directory paths. Logs and temp files go here. Default: '.'
 
         logging_level : int
-             Logging level as defined in the logging module. Default: logging.INFO (20)
+             Logging level as defined in the logging module. Default: logging.INFO
 
         poll_period : int
              The main thread polling period, in milliseconds. Default: 10ms
@@ -156,6 +129,7 @@ class Interchange(object):
         os.makedirs(self.logdir, exist_ok=True)
 
         start_file_logger("{}/interchange.log".format(self.logdir), level=logging_level)
+        logger.propagate = False
         logger.debug("Initializing Interchange process")
 
         self.client_address = client_address
@@ -167,7 +141,6 @@ class Interchange(object):
         self.context = zmq.Context()
         self.task_incoming = self.context.socket(zmq.DEALER)
         self.task_incoming.set_hwm(0)
-        self.task_incoming.RCVTIMEO = 10  # in milliseconds
         self.task_incoming.connect("tcp://{}:{}".format(client_address, client_ports[0]))
         self.results_outgoing = self.context.socket(zmq.DEALER)
         self.results_outgoing.set_hwm(0)
@@ -178,15 +151,10 @@ class Interchange(object):
         self.command_channel.connect("tcp://{}:{}".format(client_address, client_ports[2]))
         logger.info("Connected to client")
 
-        self.monitoring_enabled = False
-        if hub_address and hub_port:
-            self.hub_channel = self.context.socket(zmq.DEALER)
-            self.hub_channel.set_hwm(0)
-            self.hub_channel.connect("tcp://{}:{}".format(hub_address, hub_port))
-            self.monitoring_enabled = True
-            logger.info("Monitoring enabled and connected to hub")
+        self.hub_address = hub_address
+        self.hub_port = hub_port
 
-        self.pending_task_queue = queue.Queue(maxsize=10 ** 6)
+        self.pending_task_queue: queue.Queue[Any] = queue.Queue(maxsize=10 ** 6)
 
         self.worker_ports = worker_ports
         self.worker_port_range = worker_port_range
@@ -214,7 +182,7 @@ class Interchange(object):
         logger.info("Bound to ports {},{} for incoming worker connections".format(
             self.worker_task_port, self.worker_result_port))
 
-        self._ready_manager_queue = {}
+        self._ready_managers: Dict[bytes, ManagerRecord] = {}
 
         self.heartbeat_threshold = heartbeat_threshold
 
@@ -252,119 +220,138 @@ class Interchange(object):
 
         return tasks
 
-    def migrate_tasks_to_internal(self, kill_event):
-        """Pull tasks from the incoming tasks 0mq pipe onto the internal
+    @wrap_with_logs(target="interchange")
+    def task_puller(self):
+        """Pull tasks from the incoming tasks zmq pipe onto the internal
         pending task queue
-
-        Parameters:
-        -----------
-        kill_event : threading.Event
-              Event to let the thread know when it is time to die.
         """
-        logger.info("[TASK_PULL_THREAD] Starting")
+        logger.info("Starting")
         task_counter = 0
-        poller = zmq.Poller()
-        poller.register(self.task_incoming, zmq.POLLIN)
 
-        while not kill_event.is_set():
+        while True:
+            logger.debug("launching recv_pyobj")
             try:
                 msg = self.task_incoming.recv_pyobj()
             except zmq.Again:
                 # We just timed out while attempting to receive
-                logger.debug("[TASK_PULL_THREAD] {} tasks in internal queue".format(self.pending_task_queue.qsize()))
+                logger.debug("zmq.Again with {} tasks in internal queue".format(self.pending_task_queue.qsize()))
                 continue
 
-            if msg == 'STOP':
-                kill_event.set()
-                break
-            else:
-                self.pending_task_queue.put(msg)
-                task_counter += 1
-                logger.debug("[TASK_PULL_THREAD] Fetched task:{}".format(task_counter))
+            logger.debug("putting message onto pending_task_queue")
+            self.pending_task_queue.put(msg)
+            task_counter += 1
+            logger.debug(f"Fetched {task_counter} tasks so far")
 
-    def _command_server(self, kill_event):
+    def _create_monitoring_channel(self):
+        if self.hub_address and self.hub_port:
+            logger.info("Connecting to monitoring")
+            hub_channel = self.context.socket(zmq.DEALER)
+            hub_channel.set_hwm(0)
+            hub_channel.connect("tcp://{}:{}".format(self.hub_address, self.hub_port))
+            logger.info("Monitoring enabled and connected to hub")
+            return hub_channel
+        else:
+            return None
+
+    def _send_monitoring_info(self, hub_channel, manager: ManagerRecord):
+        if hub_channel:
+            logger.info("Sending message {} to hub".format(manager))
+
+            d: Dict = cast(Dict, manager.copy())
+            d['timestamp'] = datetime.datetime.now()
+            d['last_heartbeat'] = datetime.datetime.fromtimestamp(d['last_heartbeat'])
+
+            hub_channel.send_pyobj((MessageType.NODE_INFO, d))
+
+    @wrap_with_logs(target="interchange")
+    def _command_server(self):
         """ Command server to run async command to the interchange
         """
-        logger.debug("[COMMAND] Command Server Starting")
-        while not kill_event.is_set():
+        logger.debug("Command Server Starting")
+
+        # Need to create a new ZMQ socket for command server thread
+        hub_channel = self._create_monitoring_channel()
+
+        reply: Any  # the type of reply depends on the command_req received (aka this needs dependent types...)
+
+        while True:
             try:
                 command_req = self.command_channel.recv_pyobj()
-                logger.debug("[COMMAND] Received command request: {}".format(command_req))
+                logger.debug("Received command request: {}".format(command_req))
                 if command_req == "OUTSTANDING_C":
                     outstanding = self.pending_task_queue.qsize()
-                    for manager in self._ready_manager_queue:
-                        outstanding += len(self._ready_manager_queue[manager]['tasks'])
+                    for manager in self._ready_managers.values():
+                        outstanding += len(manager['tasks'])
                     reply = outstanding
 
                 elif command_req == "WORKERS":
                     num_workers = 0
-                    for manager in self._ready_manager_queue:
-                        num_workers += self._ready_manager_queue[manager]['worker_count']
+                    for manager in self._ready_managers.values():
+                        num_workers += manager['worker_count']
                     reply = num_workers
+
                 elif command_req == "MANAGERS":
                     reply = []
-                    for manager in self._ready_manager_queue:
-                        resp = {'manager': manager.decode('utf-8'),
-                                'block_id': self._ready_manager_queue[manager]['block_id'],
-                                'worker_count': self._ready_manager_queue[manager]['worker_count'],
-                                'tasks': len(self._ready_manager_queue[manager]['tasks']),
-                                'active': self._ready_manager_queue[manager]['active']}
+                    for manager_id in self._ready_managers:
+                        m = self._ready_managers[manager_id]
+                        idle_since = m['idle_since']
+                        if idle_since is not None:
+                            idle_duration = time.time() - idle_since
+                        else:
+                            idle_duration = 0.0
+                        resp = {'manager': manager_id.decode('utf-8'),
+                                'block_id': m['block_id'],
+                                'worker_count': m['worker_count'],
+                                'tasks': len(m['tasks']),
+                                'idle_duration': idle_duration,
+                                'active': m['active']}
                         reply.append(resp)
 
                 elif command_req.startswith("HOLD_WORKER"):
                     cmd, s_manager = command_req.split(';')
-                    manager = s_manager.encode('utf-8')
-                    logger.info("[CMD] Received HOLD_WORKER for {}".format(manager))
-                    if manager in self._ready_manager_queue:
-                        self._ready_manager_queue[manager]['active'] = False
+                    manager_id = s_manager.encode('utf-8')
+                    logger.info("Received HOLD_WORKER for {}".format(manager_id))
+                    if manager_id in self._ready_managers:
+                        m = self._ready_managers[manager_id]
+                        m['active'] = False
                         reply = True
+                        self._send_monitoring_info(hub_channel, m)
                     else:
                         reply = False
 
-                elif command_req == "SHUTDOWN":
-                    logger.info("[CMD] Received SHUTDOWN command")
-                    kill_event.set()
-                    reply = True
-
                 else:
                     reply = None
 
-                logger.debug("[COMMAND] Reply: {}".format(reply))
+                logger.debug("Reply: {}".format(reply))
                 self.command_channel.send_pyobj(reply)
 
             except zmq.Again:
-                logger.debug("[COMMAND] is alive")
+                logger.debug("Command thread is alive")
                 continue
 
-    def start(self, poll_period=None):
-        """ Start the NeedNameQeueu
-
-        Parameters:
-        ----------
-
-        TODO: Move task receiving to a thread
+    @wrap_with_logs
+    def start(self):
+        """ Start the interchange
         """
         logger.info("Incoming ports bound")
 
-        if poll_period is None:
-            poll_period = self.poll_period
+        hub_channel = self._create_monitoring_channel()
+
+        poll_period = self.poll_period
 
         start = time.time()
         count = 0
 
         self._kill_event = threading.Event()
-        self._task_puller_thread = threading.Thread(target=self.migrate_tasks_to_internal,
-                                                    args=(self._kill_event,),
+        self._task_puller_thread = threading.Thread(target=self.task_puller,
                                                     name="Interchange-Task-Puller")
         self._task_puller_thread.start()
 
         self._command_thread = threading.Thread(target=self._command_server,
-                                                args=(self._kill_event,),
                                                 name="Interchange-Command")
         self._command_thread.start()
 
         poller = zmq.Poller()
-        # poller.register(self.task_incoming, zmq.POLLIN)
         poller.register(self.task_outgoing, zmq.POLLIN)
         poller.register(self.results_incoming, zmq.POLLIN)
 
@@ -372,49 +359,48 @@ class Interchange(object):
         # for scheduling a job (or maybe any other attention?).
         # Anything altering the state of the manager should add it
         # onto this list.
-        interesting_managers = set()
+        interesting_managers: Set[bytes] = set()
 
         while not self._kill_event.is_set():
             self.socks = dict(poller.poll(timeout=poll_period))
 
             # Listen for requests for work
             if self.task_outgoing in self.socks and self.socks[self.task_outgoing] == zmq.POLLIN:
-                logger.debug("[MAIN] starting task_outgoing section")
+                logger.debug("starting task_outgoing section")
                 message = self.task_outgoing.recv_multipart()
-                manager = message[0]
+                manager_id = message[0]
 
-                if manager not in self._ready_manager_queue:
+                if manager_id not in self._ready_managers:
                     reg_flag = False
 
                     try:
                         msg = json.loads(message[1].decode('utf-8'))
-                        msg['reg_time'] = datetime.datetime.strptime(msg['reg_time'], "%Y-%m-%d %H:%M:%S")
                         reg_flag = True
                     except Exception:
-                        logger.warning("[MAIN] Got Exception reading registration message from manager: {}".format(
-                            manager), exc_info=True)
-                        logger.debug("[MAIN] Message :\n{}\n".format(message[0]))
+                        logger.warning("Got Exception reading registration message from manager: {}".format(
+                            manager_id), exc_info=True)
+                        logger.debug("Message: \n{}\n".format(message[1]))
                     else:
                         # We set up an entry only if registration works correctly
-                        self._ready_manager_queue[manager] = {'last': time.time(),
-                                                              'free_capacity': 0,
-                                                              'block_id': None,
-                                                              'max_capacity': 0,
-                                                              'worker_count': 0,
-                                                              'active': True,
-                                                              'tasks': []}
+                        self._ready_managers[manager_id] = {'last_heartbeat': time.time(),
+                                                            'idle_since': time.time(),
+                                                            'free_capacity': 0,
+                                                            'block_id': None,
+                                                            'max_capacity': 0,
+                                                            'worker_count': 0,
+                                                            'active': True,
+                                                            'tasks': []}
                     if reg_flag is True:
-                        interesting_managers.add(manager)
-                        logger.info("[MAIN] Adding manager: {} to ready queue".format(manager))
-                        self._ready_manager_queue[manager].update(msg)
-                        logger.info("[MAIN] Registration info for manager {}: {}".format(manager, msg))
-                        if self.monitoring_enabled:
-                            logger.info("Sending message {} to hub".format(self._ready_manager_queue[manager]))
-                            self.hub_channel.send_pyobj((MessageType.NODE_INFO, self._ready_manager_queue[manager]))
+                        interesting_managers.add(manager_id)
+                        logger.info("Adding manager: {} to ready queue".format(manager_id))
+                        m = self._ready_managers[manager_id]
+                        m.update(msg)
+                        logger.info("Registration info for manager {}: {}".format(manager_id, msg))
+                        self._send_monitoring_info(hub_channel, m)
 
                         if (msg['python_v'].rsplit(".", 1)[0] != self.current_platform['python_v'].rsplit(".", 1)[0] or
                             msg['parsl_v'] != self.current_platform['parsl_v']):
-                            logger.warning("[MAIN] Manager {} has incompatible version info with the interchange".format(manager))
+                            logger.error("Manager {} has incompatible version info with the interchange".format(manager_id))
                             logger.debug("Setting kill event")
                             self._kill_event.set()
                             e = VersionMismatch("py.v={} parsl.v={}".format(self.current_platform['python_v'].rsplit(".", 1)[0],
@@ -422,35 +408,35 @@ class Interchange(object):
                                                 "py.v={} parsl.v={}".format(msg['python_v'].rsplit(".", 1)[0],
                                                                             msg['parsl_v'])
                             )
-                            result_package = {'task_id': -1, 'exception': serialize_object(e)}
+                            result_package = {'type': 'result', 'task_id': -1, 'exception': serialize_object(e)}
                             pkl_package = pickle.dumps(result_package)
                             self.results_outgoing.send(pkl_package)
-                            logger.warning("[MAIN] Sent failure reports, unregistering manager")
+                            logger.error("Sent failure reports, shutting down interchange")
                         else:
-                            logger.info("[MAIN] Manager {} has compatible Parsl version {}".format(manager, msg['parsl_v']))
-                            logger.info("[MAIN] Manager {} has compatible Python version {}".format(manager,
-                                                                                                    msg['python_v'].rsplit(".", 1)[0]))
+                            logger.info("Manager {} has compatible Parsl version {}".format(manager_id, msg['parsl_v']))
+                            logger.info("Manager {} has compatible Python version {}".format(manager_id,
+                                                                                             msg['python_v'].rsplit(".", 1)[0]))
                     else:
                         # Registration has failed.
-                        logger.debug("[MAIN] Suppressing bad registration from manager:{}".format(
-                            manager))
+                        logger.debug("Suppressing bad registration from manager: {}".format(
+                            manager_id))
 
                 else:
                     tasks_requested = int.from_bytes(message[1], "little")
-                    self._ready_manager_queue[manager]['last'] = time.time()
+                    self._ready_managers[manager_id]['last_heartbeat'] = time.time()
                     if tasks_requested == HEARTBEAT_CODE:
-                        logger.debug("[MAIN] Manager {} sent heartbeat".format(manager))
-                        self.task_outgoing.send_multipart([manager, b'', PKL_HEARTBEAT_CODE])
+                        logger.debug("Manager {} sent heartbeat via tasks connection".format(manager_id))
+                        self.task_outgoing.send_multipart([manager_id, b'', PKL_HEARTBEAT_CODE])
                     else:
-                        logger.debug("[MAIN] Manager {} requested {} tasks".format(manager, tasks_requested))
-                        self._ready_manager_queue[manager]['free_capacity'] = tasks_requested
-                        interesting_managers.add(manager)
-                logger.debug("[MAIN] leaving task_outgoing section")
+                        logger.debug("Manager {} requested {} tasks".format(manager_id, tasks_requested))
+                        self._ready_managers[manager_id]['free_capacity'] = tasks_requested
+                        interesting_managers.add(manager_id)
+                logger.debug("leaving task_outgoing section")
 
             # If we had received any requests, check if there are tasks that could be passed
 
             logger.debug("Managers count (interesting/total): {interesting}/{total}".format(
-                total=len(self._ready_manager_queue),
+                total=len(self._ready_managers),
                 interesting=len(interesting_managers)))
 
             if interesting_managers and not self.pending_task_queue.empty():
@@ -458,73 +444,108 @@ class Interchange(object):
                 random.shuffle(shuffled_managers)
 
                 while shuffled_managers and not self.pending_task_queue.empty():  # cf. the if statement above...
-                    manager = shuffled_managers.pop()
-                    tasks_inflight = len(self._ready_manager_queue[manager]['tasks'])
-                    real_capacity = min(self._ready_manager_queue[manager]['free_capacity'],
-                                        self._ready_manager_queue[manager]['max_capacity'] - tasks_inflight)
+                    manager_id = shuffled_managers.pop()
+                    m = self._ready_managers[manager_id]
+                    tasks_inflight = len(m['tasks'])
+                    real_capacity = min(m['free_capacity'],
+                                        m['max_capacity'] - tasks_inflight)
 
-                    if (real_capacity and self._ready_manager_queue[manager]['active']):
+                    if (real_capacity and m['active']):
                         tasks = self.get_tasks(real_capacity)
                         if tasks:
-                            self.task_outgoing.send_multipart([manager, b'', pickle.dumps(tasks)])
+                            self.task_outgoing.send_multipart([manager_id, b'', pickle.dumps(tasks)])
                             task_count = len(tasks)
                             count += task_count
                             tids = [t['task_id'] for t in tasks]
-                            self._ready_manager_queue[manager]['free_capacity'] -= task_count
-                            self._ready_manager_queue[manager]['tasks'].extend(tids)
-                            logger.debug("[MAIN] Sent tasks: {} to manager {}".format(tids, manager))
-                            if self._ready_manager_queue[manager]['free_capacity'] > 0:
-                                logger.debug("[MAIN] Manager {} has free_capacity {}".format(manager, self._ready_manager_queue[manager]['free_capacity']))
+                            m['free_capacity'] -= task_count
+                            m['tasks'].extend(tids)
+                            m['idle_since'] = None
+                            logger.debug("Sent tasks: {} to manager {}".format(tids, manager_id))
+                            if m['free_capacity'] > 0:
+                                logger.debug("Manager {} has free_capacity {}".format(manager_id, m['free_capacity']))
                                 # ... so keep it in the interesting_managers list
                             else:
-                                logger.debug("[MAIN] Manager {} is now saturated".format(manager))
-                                interesting_managers.remove(manager)
+                                logger.debug("Manager {} is now saturated".format(manager_id))
+                                interesting_managers.remove(manager_id)
                     else:
-                        interesting_managers.remove(manager)
-                        # logger.debug("Nothing to send to manager {}".format(manager))
-                logger.debug("[MAIN] leaving _ready_manager_queue section, with {} managers still interesting".format(len(interesting_managers)))
+                        interesting_managers.remove(manager_id)
+                        # logger.debug("Nothing to send to manager {}".format(manager_id))
+                logger.debug("leaving _ready_managers section, with {} managers still interesting".format(len(interesting_managers)))
             else:
-                logger.debug("[MAIN] either no interesting managers or no tasks, so skipping manager pass")
+                logger.debug("either no interesting managers or no tasks, so skipping manager pass")
             # Receive any results and forward to client
             if self.results_incoming in self.socks and self.socks[self.results_incoming] == zmq.POLLIN:
-                logger.debug("[MAIN] entering results_incoming section")
-                manager, *b_messages = self.results_incoming.recv_multipart()
-                if manager not in self._ready_manager_queue:
-                    logger.warning("[MAIN] Received a result from a un-registered manager: {}".format(manager))
+                logger.debug("entering results_incoming section")
+                manager_id, *all_messages = self.results_incoming.recv_multipart()
+                if manager_id not in self._ready_managers:
+                    logger.warning("Received a result from a un-registered manager: {}".format(manager_id))
                 else:
-                    logger.debug("[MAIN] Got {} result items in batch".format(len(b_messages)))
-                    for b_message in b_messages:
-                        r = pickle.loads(b_message)
-                        try:
-                            self._ready_manager_queue[manager]['tasks'].remove(r['task_id'])
-                        except Exception:
-                            # If we reach here, there's something very wrong.
-                            logger.exception("Ignoring exception removing task_id {} for manager {} with task list {}".format(
-                                r['task_id'],
-                                manager,
-                                self._ready_manager_queue[manager]['tasks']))
+                    logger.debug(f"Got {len(all_messages)} result items in batch from manager {manager_id}")
 
-                    self.results_outgoing.send_multipart(b_messages)
-                    logger.debug("[MAIN] Current tasks: {}".format(self._ready_manager_queue[manager]['tasks']))
-                logger.debug("[MAIN] leaving results_incoming section")
+                    b_messages = []
 
-            bad_managers = [manager for manager in self._ready_manager_queue if
-                            time.time() - self._ready_manager_queue[manager]['last'] > self.heartbeat_threshold]
-            for manager in bad_managers:
-                logger.debug("[MAIN] Last: {} Current: {}".format(self._ready_manager_queue[manager]['last'], time.time()))
-                logger.warning("[MAIN] Too many heartbeats missed for manager {}".format(manager))
+                    for p_message in all_messages:
+                        r = pickle.loads(p_message)
+                        if r['type'] == 'result':
+                            # process this for task ID and forward to executor
+                            b_messages.append((p_message, r))
+                        elif r['type'] == 'monitoring':
+                            hub_channel.send_pyobj(r['payload'])
+                        elif r['type'] == 'heartbeat':
+                            logger.debug(f"Manager {manager_id} sent heartbeat via results connection")
+                            b_messages.append((p_message, r))
+                        else:
+                            logger.error("Interchange discarding result_queue message of unknown type: {}".format(r['type']))
 
-                for tid in self._ready_manager_queue[manager]['tasks']:
+                    m = self._ready_managers[manager_id]
+                    for (b_message, r) in b_messages:
+                        assert 'type' in r, f"Message is missing type entry: {r}"
+                        if r['type'] == 'result':
+                            try:
+                                logger.debug(f"Removing task {r['task_id']} from manager record {manager_id}")
+                                m['tasks'].remove(r['task_id'])
+                            except Exception:
+                                # If we reach here, there's something very wrong.
+                                logger.exception("Ignoring exception removing task_id {} for manager {} with task list {}".format(
+                                    r['task_id'],
+                                    manager_id,
+                                    m['tasks']))
+
+                    b_messages_to_send = []
+                    for (b_message, _) in b_messages:
+                        b_messages_to_send.append(b_message)
+
+                    if b_messages_to_send:
+                        logger.debug("Sending messages on results_outgoing")
+                        self.results_outgoing.send_multipart(b_messages_to_send)
+                        logger.debug("Sent messages on results_outgoing")
+
+                    logger.debug(f"Current tasks on manager {manager_id}: {m['tasks']}")
+                    if len(m['tasks']) == 0 and m['idle_since'] is None:
+                        m['idle_since'] = time.time()
+                logger.debug("leaving results_incoming section")
+
+            bad_managers = [(manager_id, m) for (manager_id, m) in self._ready_managers.items() if
+                            time.time() - m['last_heartbeat'] > self.heartbeat_threshold]
+            for (manager_id, m) in bad_managers:
+                logger.debug("Last: {} Current: {}".format(m['last_heartbeat'], time.time()))
+                logger.warning(f"Too many heartbeats missed for manager {manager_id} - removing manager")
+                if m['active']:
+                    m['active'] = False
+                    self._send_monitoring_info(hub_channel, m)
+
+                logger.warning(f"Cancelling htex tasks {m['tasks']} on removed manager")
+                for tid in m['tasks']:
                     try:
-                        raise ManagerLost(manager, self._ready_manager_queue[manager]['hostname'])
+                        raise ManagerLost(manager_id, m['hostname'])
                     except Exception:
-                        result_package = {'task_id': tid, 'exception': serialize_object(RemoteExceptionWrapper(*sys.exc_info()))}
+                        result_package = {'type': 'result', 'task_id': tid, 'exception': serialize_object(RemoteExceptionWrapper(*sys.exc_info()))}
                         pkl_package = pickle.dumps(result_package)
                         self.results_outgoing.send(pkl_package)
-                        logger.warning("[MAIN] Sent failure reports, unregistering manager")
-                self._ready_manager_queue.pop(manager, 'None')
-                if manager in interesting_managers:
-                    interesting_managers.remove(manager)
+                logger.warning("Sent failure reports, unregistering manager")
+                self._ready_managers.pop(manager_id, 'None')
+                if manager_id in interesting_managers:
+                    interesting_managers.remove(manager_id)
 
         delta = time.time() - start
         logger.info("Processed {} tasks in {} seconds".format(count, delta))
@@ -552,7 +573,7 @@ def start_file_logger(filename, name='interchange', level=logging.DEBUG, format_
         None.
     """
     if format_string is None:
-        format_string = "%(asctime)s.%(msecs)03d %(name)s:%(lineno)d [%(levelname)s]  %(message)s"
+        format_string = "%(asctime)s.%(msecs)03d %(name)s:%(lineno)d %(processName)s(%(process)d) %(threadName)s %(funcName)s [%(levelname)s]  %(message)s"
 
     global logger
     logger = logging.getLogger(name)
@@ -564,11 +585,13 @@ def start_file_logger(filename, name='interchange', level=logging.DEBUG, format_
     logger.addHandler(handler)
 
 
+@wrap_with_logs(target="interchange")
 def starter(comm_q, *args, **kwargs):
     """Start the interchange process
 
     The executor is expected to call this function. The args, kwargs match that of the Interchange.__init__
     """
+    setproctitle("parsl: HTEX interchange")
     # logger = multiprocessing.get_logger()
     ic = Interchange(*args, **kwargs)
     comm_q.put((ic.worker_task_port,
