@@ -5,33 +5,30 @@ import logging
 import os
 import sys
 import platform
-# import random
 import threading
 import pickle
 import time
-import datetime
 import queue
 import uuid
+from typing import Sequence, Optional
+
 import zmq
 import math
 import json
 import psutil
 import multiprocessing
 
+from parsl.process_loggers import wrap_with_logs
+
 from parsl.version import VERSION as PARSL_VERSION
 from parsl.app.errors import RemoteExceptionWrapper
 from parsl.executors.high_throughput.errors import WorkerLost
 from parsl.executors.high_throughput.probe import probe_addresses
-if platform.system() == 'Darwin':
-    from parsl.executors.high_throughput.mac_safe_queue import MacSafeQueue as mpQueue
-else:
-    from multiprocessing import Queue as mpQueue
+from parsl.multiprocessing import ForkProcess as mpProcess
 
-from ipyparallel.serialize import unpack_apply_message  # pack_apply_message,
-from ipyparallel.serialize import serialize_object
+from parsl.multiprocessing import SizedQueue as mpQueue
 
-RESULT_TAG = 10
-TASK_REQUEST_TAG = 11
+from parsl.serialize import unpack_apply_message, serialize
 
 HEARTBEAT_CODE = (2 ** 32) - 1
 
@@ -39,7 +36,7 @@ HEARTBEAT_CODE = (2 ** 32) - 1
 class Manager(object):
     """ Manager manages task execution by the workers
 
-                |         0mq              |    Manager         |   Worker Processes
+                |         zmq              |    Manager         |   Worker Processes
                 |                          |                    |
                 | <-----Request N task-----+--Count task reqs   |      Request task<--+
     Interchange | -------------------------+->Receive task batch|          |          |
@@ -65,7 +62,9 @@ class Manager(object):
                  block_id=None,
                  heartbeat_threshold=120,
                  heartbeat_period=30,
-                 poll_period=10):
+                 poll_period=10,
+                 cpu_affinity=False,
+                 available_accelerators: Sequence[str] = ()):
         """
         Parameters
         ----------
@@ -76,9 +75,6 @@ class Manager(object):
              Timeout in seconds for the address probe to detect viable addresses
              to the interchange. Default : 30s
 
-        worker_url : str
-             Worker url on which workers will attempt to connect back
-
         uid : str
              string unique identifier
 
@@ -113,10 +109,17 @@ class Manager(object):
              assumes that the interchange is lost and the manager shuts down. Default:120
 
         heartbeat_period : int
-             Number of seconds after which a heartbeat message is sent to the interchange
+             Number of seconds after which a heartbeat message is sent to the interchange, and workers
+             are checked for liveness.
 
         poll_period : int
              Timeout period used by the manager in milliseconds. Default: 10ms
+
+        cpu_affinity : str
+             Whether each worker should force its affinity to different CPUs
+
+        available_accelerators: list of str
+            List of accelerators available to the workers. Default: Empty list
         """
 
         logger.info("Manager started")
@@ -148,7 +151,7 @@ class Manager(object):
         self.result_outgoing.setsockopt(zmq.IDENTITY, uid.encode('utf-8'))
         self.result_outgoing.setsockopt(zmq.LINGER, 0)
         self.result_outgoing.connect(result_q_url)
-        logger.info("Manager connected")
+        logger.info("Manager connected to interchange")
 
         self.uid = uid
         self.block_id = block_id
@@ -174,7 +177,6 @@ class Manager(object):
         self.worker_count = min(max_workers,
                                 mem_slots,
                                 math.floor(cores_on_node / cores_per_worker))
-        logger.info("Manager will spawn {} workers".format(self.worker_count))
 
         self.pending_task_queue = mpQueue()
         self.pending_result_queue = mpQueue()
@@ -187,6 +189,14 @@ class Manager(object):
         self.heartbeat_period = heartbeat_period
         self.heartbeat_threshold = heartbeat_threshold
         self.poll_period = poll_period
+        self.cpu_affinity = cpu_affinity
+
+        # Define accelerator available, adjust worker count accordingly
+        self.available_accelerators = available_accelerators
+        self.accelerators_available = len(available_accelerators) > 0
+        if self.accelerators_available:
+            self.worker_count = min(len(self.available_accelerators), self.worker_count)
+        logger.info("Manager will spawn {} workers".format(self.worker_count))
 
     def create_reg_message(self):
         """ Creates a registration message to identify the worker to the interchange
@@ -196,6 +206,7 @@ class Manager(object):
                                              sys.version_info.minor,
                                              sys.version_info.micro),
                'worker_count': self.worker_count,
+               'uid': self.uid,
                'block_id': self.block_id,
                'prefetch_capacity': self.prefetch_capacity,
                'max_capacity': self.worker_count + self.prefetch_capacity,
@@ -204,20 +215,20 @@ class Manager(object):
                'dir': os.getcwd(),
                'cpu_count': psutil.cpu_count(logical=False),
                'total_memory': psutil.virtual_memory().total,
-               'reg_time': datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
         }
         b_msg = json.dumps(msg).encode('utf-8')
         return b_msg
 
-    def heartbeat(self):
+    def heartbeat_to_incoming(self):
         """ Send heartbeat to the incoming task queue
         """
         heartbeat = (HEARTBEAT_CODE).to_bytes(4, "little")
-        r = self.task_incoming.send(heartbeat)
-        logger.debug("Return from heartbeat: {}".format(r))
+        self.task_incoming.send(heartbeat)
+        logger.debug("Sent heartbeat")
 
+    @wrap_with_logs
     def pull_tasks(self, kill_event):
-        """ Pull tasks from the incoming tasks 0mq pipe onto the internal
+        """ Pull tasks from the incoming tasks zmq pipe onto the internal
         pending task queue
 
         Parameters:
@@ -225,7 +236,7 @@ class Manager(object):
         kill_event : threading.Event
               Event to let the thread know when it is time to die.
         """
-        logger.info("[TASK PULL THREAD] starting")
+        logger.info("starting")
         poller = zmq.Poller()
         poller.register(self.task_incoming, zmq.POLLIN)
 
@@ -243,15 +254,15 @@ class Manager(object):
             ready_worker_count = self.ready_worker_queue.qsize()
             pending_task_count = self.pending_task_queue.qsize()
 
-            logger.debug("[TASK_PULL_THREAD] ready workers:{}, pending tasks:{}".format(ready_worker_count,
-                                                                                        pending_task_count))
+            logger.debug("ready workers: {}, pending tasks: {}".format(ready_worker_count,
+                                                                       pending_task_count))
 
             if time.time() > last_beat + self.heartbeat_period:
-                self.heartbeat()
+                self.heartbeat_to_incoming()
                 last_beat = time.time()
 
             if pending_task_count < self.max_queue_size and ready_worker_count > 0:
-                logger.debug("[TASK_PULL_THREAD] Requesting tasks: {}".format(ready_worker_count))
+                logger.debug("Requesting tasks: {}".format(ready_worker_count))
                 msg = ((ready_worker_count).to_bytes(4, "little"))
                 self.task_incoming.send(msg)
 
@@ -264,7 +275,7 @@ class Manager(object):
                 last_interchange_contact = time.time()
 
                 if tasks == 'STOP':
-                    logger.critical("[TASK_PULL_THREAD] Received stop request")
+                    logger.critical("Received stop request")
                     kill_event.set()
                     break
 
@@ -273,16 +284,15 @@ class Manager(object):
 
                 else:
                     task_recv_counter += len(tasks)
-                    logger.debug("[TASK_PULL_THREAD] Got tasks: {} of {}".format([t['task_id'] for t in tasks],
-                                                                                 task_recv_counter))
+                    logger.debug("Got tasks: {}, cumulative count of tasks: {}".format([t['task_id'] for t in tasks], task_recv_counter))
 
                     for task in tasks:
                         self.pending_task_queue.put(task)
-                        # logger.debug("[TASK_PULL_THREAD] Ready tasks: {}".format(
+                        # logger.debug("Ready tasks: {}".format(
                         #    [i['task_id'] for i in self.pending_task_queue]))
 
             else:
-                logger.debug("[TASK_PULL_THREAD] No incoming tasks")
+                logger.debug("No incoming tasks")
                 # Limit poll duration to heartbeat_period
                 # heartbeat_period is in s vs poll_timer in ms
                 if not poll_timer:
@@ -291,13 +301,14 @@ class Manager(object):
 
                 # Only check if no messages were received.
                 if time.time() > last_interchange_contact + self.heartbeat_threshold:
-                    logger.critical("[TASK_PULL_THREAD] Missing contact with interchange beyond heartbeat_threshold")
+                    logger.critical("Missing contact with interchange beyond heartbeat_threshold")
                     kill_event.set()
-                    logger.critical("[TASK_PULL_THREAD] Exiting")
+                    logger.critical("Exiting")
                     break
 
+    @wrap_with_logs
     def push_results(self, kill_event):
-        """ Listens on the pending_result_queue and sends out results via 0mq
+        """ Listens on the pending_result_queue and sends out results via zmq
 
         Parameters:
         -----------
@@ -305,35 +316,48 @@ class Manager(object):
               Event to let the thread know when it is time to die.
         """
 
-        logger.debug("[RESULT_PUSH_THREAD] Starting thread")
+        logger.debug("Starting result push thread")
 
         push_poll_period = max(10, self.poll_period) / 1000    # push_poll_period must be atleast 10 ms
-        logger.debug("[RESULT_PUSH_THREAD] push poll period: {}".format(push_poll_period))
+        logger.debug("push poll period: {}".format(push_poll_period))
 
         last_beat = time.time()
+        last_result_beat = time.time()
         items = []
 
         while not kill_event.is_set():
-
             try:
+                logger.debug("Starting pending_result_queue get")
                 r = self.pending_result_queue.get(block=True, timeout=push_poll_period)
+                logger.debug("Got a result item")
                 items.append(r)
             except queue.Empty:
-                pass
+                logger.debug("pending_result_queue get timeout without result item")
             except Exception as e:
-                logger.exception("[RESULT_PUSH_THREAD] Got an exception: {}".format(e))
+                logger.exception("Got an exception: {}".format(e))
+
+            if time.time() > last_result_beat + self.heartbeat_period:
+                logger.info(f"Sending heartbeat via results connection: last_result_beat={last_result_beat} heartbeat_period={self.heartbeat_period} seconds")
+                last_result_beat = time.time()
+                items.append(pickle.dumps({'type': 'heartbeat'}))
 
-            # If we have reached poll_period duration or timer has expired, we send results
             if len(items) >= self.max_queue_size or time.time() > last_beat + push_poll_period:
                 last_beat = time.time()
                 if items:
+                    logger.debug(f"Result send: Pushing {len(items)} items")
                     self.result_outgoing.send_multipart(items)
+                    logger.debug("Result send: Pushed")
                     items = []
+                else:
+                    logger.debug("Result send: No items to push")
+            else:
+                logger.debug(f"Result send: check condition not met - deferring {len(items)} result items")
 
-        logger.critical("[RESULT_PUSH_THREAD] Exiting")
+        logger.critical("Exiting")
 
+    @wrap_with_logs
     def worker_watchdog(self, kill_event):
-        """ Listens on the pending_result_queue and sends out results via 0mq
+        """Keeps workers alive.
 
         Parameters:
         -----------
@@ -341,38 +365,39 @@ class Manager(object):
               Event to let the thread know when it is time to die.
         """
 
-        logger.debug("[WORKER_WATCHDOG_THREAD] Starting thread")
+        logger.debug("Starting worker watchdog")
 
         while not kill_event.is_set():
             for worker_id, p in self.procs.items():
                 if not p.is_alive():
-                    logger.info("[WORKER_WATCHDOG_THREAD] Worker {} has died".format(worker_id))
+                    logger.info("Worker {} has died".format(worker_id))
                     try:
                         task = self._tasks_in_progress.pop(worker_id)
-                        logger.info("[WORKER_WATCHDOG_THREAD] Worker {} was busy when it died".format(worker_id))
+                        logger.info("Worker {} was busy when it died".format(worker_id))
                         try:
                             raise WorkerLost(worker_id, platform.node())
                         except Exception:
-                            logger.info("[WORKER_WATCHDOG_THREAD] Putting exception for task {} in the pending result queue".format(task['task_id']))
-                            result_package = {'task_id': task['task_id'], 'exception': serialize_object(RemoteExceptionWrapper(*sys.exc_info()))}
+                            logger.info("Putting exception for task {} in the pending result queue".format(task['task_id']))
+                            result_package = {'type': 'result', 'task_id': task['task_id'], 'exception': serialize(RemoteExceptionWrapper(*sys.exc_info()))}
                             pkl_package = pickle.dumps(result_package)
                             self.pending_result_queue.put(pkl_package)
                     except KeyError:
-                        logger.info("[WORKER_WATCHDOG_THREAD] Worker {} was not busy when it died".format(worker_id))
+                        logger.info("Worker {} was not busy when it died".format(worker_id))
 
-                    p = multiprocessing.Process(target=worker, args=(worker_id,
-                                                                     self.uid,
-                                                                     self.worker_count,
-                                                                     self.pending_task_queue,
-                                                                     self.pending_result_queue,
-                                                                     self.ready_worker_queue,
-                                                                     self._tasks_in_progress
-                                                                 ), name="HTEX-Worker-{}".format(worker_id))
+                    p = mpProcess(target=worker, args=(worker_id,
+                                                       self.uid,
+                                                       self.worker_count,
+                                                       self.pending_task_queue,
+                                                       self.pending_result_queue,
+                                                       self.ready_worker_queue,
+                                                       self._tasks_in_progress,
+                                                       self.cpu_affinity
+                                                 ), name="HTEX-Worker-{}".format(worker_id))
                     self.procs[worker_id] = p
-                    logger.info("[WORKER_WATCHDOG_THREAD] Worker {} has been restarted".format(worker_id))
-                time.sleep(self.poll_period)
+                    logger.info("Worker {} has been restarted".format(worker_id))
+                time.sleep(self.heartbeat_period)
 
-        logger.critical("[WORKER_WATCHDOG_THREAD] Exiting")
+        logger.critical("Exiting")
 
     def start(self):
         """ Start the worker processes.
@@ -385,18 +410,20 @@ class Manager(object):
 
         self.procs = {}
         for worker_id in range(self.worker_count):
-            p = multiprocessing.Process(target=worker, args=(worker_id,
-                                                             self.uid,
-                                                             self.worker_count,
-                                                             self.pending_task_queue,
-                                                             self.pending_result_queue,
-                                                             self.ready_worker_queue,
-                                                             self._tasks_in_progress
-                                                         ), name="HTEX-Worker-{}".format(worker_id))
+            p = mpProcess(target=worker, args=(worker_id,
+                                               self.uid,
+                                               self.worker_count,
+                                               self.pending_task_queue,
+                                               self.pending_result_queue,
+                                               self.ready_worker_queue,
+                                               self._tasks_in_progress,
+                                               self.cpu_affinity,
+                                               self.available_accelerators[worker_id] if self.accelerators_available else None),
+                          name="HTEX-Worker-{}".format(worker_id))
             p.start()
             self.procs[worker_id] = p
 
-        logger.debug("Manager synced with workers")
+        logger.debug("Workers started")
 
         self._task_puller_thread = threading.Thread(target=self.pull_tasks,
                                                     args=(self._kill_event,),
@@ -416,17 +443,17 @@ class Manager(object):
         # TODO : Add mechanism in this loop to stop the worker pool
         # This might need a multiprocessing event to signal back.
         self._kill_event.wait()
-        logger.critical("[MAIN] Received kill event, terminating worker processes")
+        logger.critical("Received kill event, terminating worker processes")
 
         self._task_puller_thread.join()
         self._result_pusher_thread.join()
         self._worker_watchdog_thread.join()
         for proc_id in self.procs:
             self.procs[proc_id].terminate()
-            logger.critical("Terminating worker {}:{}".format(self.procs[proc_id],
-                                                              self.procs[proc_id].is_alive()))
+            logger.critical("Terminating worker {}: is_alive()={}".format(self.procs[proc_id],
+                                                                          self.procs[proc_id].is_alive()))
             self.procs[proc_id].join()
-            logger.debug("Worker:{} joined successfully".format(self.procs[proc_id]))
+            logger.debug("Worker {} joined successfully".format(self.procs[proc_id]))
 
         self.task_incoming.close()
         self.result_outgoing.close()
@@ -461,20 +488,12 @@ def execute_task(bufs):
 
     code = "{0} = {1}(*{2}, **{3})".format(resultname, fname,
                                            argname, kwargname)
-    try:
-        # logger.debug("[RUNNER] Executing: {0}".format(code))
-        exec(code, user_ns, user_ns)
-
-    except Exception as e:
-        logger.warning("Caught exception; will raise it: {}".format(e), exc_info=True)
-        raise e
-
-    else:
-        # logger.debug("[RUNNER] Result: {0}".format(user_ns.get(resultname)))
-        return user_ns.get(resultname)
+    exec(code, user_ns, user_ns)
+    return user_ns.get(resultname)
 
 
-def worker(worker_id, pool_id, pool_size, task_queue, result_queue, worker_queue, tasks_in_progress):
+@wrap_with_logs(target="worker_log")
+def worker(worker_id, pool_id, pool_size, task_queue, result_queue, worker_queue, tasks_in_progress, cpu_affinity, accelerator: Optional[str]):
     """
 
     Put request token into queue
@@ -482,21 +501,56 @@ def worker(worker_id, pool_id, pool_size, task_queue, result_queue, worker_queue
     Pop request from queue
     Put result into result_queue
     """
-    start_file_logger('{}/block-{}/{}/worker_{}.log'.format(args.logdir, args.block_id, pool_id, worker_id),
-                      worker_id,
-                      name="worker_log",
-                      level=logging.DEBUG if args.debug else logging.INFO)
+
+    # override the global logger inherited from the __main__ process (which
+    # usually logs to manager.log) with one specific to this worker.
+    global logger
+    logger = start_file_logger('{}/block-{}/{}/worker_{}.log'.format(args.logdir, args.block_id, pool_id, worker_id),
+                               worker_id,
+                               name="worker_log",
+                               level=logging.DEBUG if args.debug else logging.INFO)
 
     # Store worker ID as an environment variable
     os.environ['PARSL_WORKER_RANK'] = str(worker_id)
     os.environ['PARSL_WORKER_COUNT'] = str(pool_size)
     os.environ['PARSL_WORKER_POOL_ID'] = str(pool_id)
+    os.environ['PARSL_WORKER_BLOCK_ID'] = str(args.block_id)
+
+    # share the result queue with monitoring code so it too can send results down that channel
+    import parsl.executors.high_throughput.monitoring_info as mi
+    mi.result_queue = result_queue
 
     # Sync worker with master
     logger.info('Worker {} started'.format(worker_id))
     if args.debug:
         logger.debug("Debug logging enabled")
 
+    # If desired, set process affinity
+    if cpu_affinity != "none":
+        # Count the number of cores per worker
+        avail_cores = sorted(os.sched_getaffinity(0))  # Get the available processors
+        cores_per_worker = len(avail_cores) // pool_size
+        assert cores_per_worker > 0, "Affinity does not work if there are more workers than cores"
+
+        # Determine this worker's cores
+        if cpu_affinity == "block":
+            my_cores = avail_cores[cores_per_worker * worker_id:cores_per_worker * (worker_id + 1)]
+        elif cpu_affinity == "alternating":
+            my_cores = avail_cores[worker_id::pool_size]
+        else:
+            raise ValueError("Affinity strategy {} is not supported".format(cpu_affinity))
+
+        # Set the affinity for this worker
+        os.sched_setaffinity(0, my_cores)
+        logger.info("Set worker CPU affinity to {}".format(my_cores))
+
+    # If desired, pin to accelerator
+    if accelerator is not None:
+        os.environ["CUDA_VISIBLE_DEVICES"] = accelerator
+        os.environ["ROCR_VISIBLE_DEVICES"] = accelerator
+        os.environ["SYCL_DEVICE_FILTER"] = f"*:*:{accelerator}"
+        logger.info(f'Pinned worker to accelerator: {accelerator}')
+
     while True:
         worker_queue.put(worker_id)
 
@@ -514,12 +568,12 @@ def worker(worker_id, pool_id, pool_size, task_queue, result_queue, worker_queue
 
         try:
             result = execute_task(req['buffer'])
-            serialized_result = serialize_object(result, buffer_threshold=1e6)
+            serialized_result = serialize(result, buffer_threshold=1e6)
         except Exception as e:
             logger.info('Caught an exception: {}'.format(e))
-            result_package = {'task_id': tid, 'exception': serialize_object(RemoteExceptionWrapper(*sys.exc_info()))}
+            result_package = {'type': 'result', 'task_id': tid, 'exception': serialize(RemoteExceptionWrapper(*sys.exc_info()))}
         else:
-            result_package = {'task_id': tid, 'result': serialized_result}
+            result_package = {'type': 'result', 'task_id': tid, 'result': serialized_result}
             # logger.debug("Result: {}".format(result))
 
         logger.info("Completed task {}".format(tid))
@@ -527,13 +581,13 @@ def worker(worker_id, pool_id, pool_size, task_queue, result_queue, worker_queue
             pkl_package = pickle.dumps(result_package)
         except Exception:
             logger.exception("Caught exception while trying to pickle the result package")
-            pkl_package = pickle.dumps({'task_id': tid,
-                                        'exception': serialize_object(
-                                            RemoteExceptionWrapper(*sys.exc_info()))
+            pkl_package = pickle.dumps({'type': 'result', 'task_id': tid,
+                                        'exception': serialize(RemoteExceptionWrapper(*sys.exc_info()))
             })
 
         result_queue.put(pkl_package)
         tasks_in_progress.pop(worker_id)
+        logger.info("All processing finished for task {}".format(tid))
 
 
 def start_file_logger(filename, rank, name='parsl', level=logging.DEBUG, format_string=None):
@@ -549,9 +603,10 @@ def start_file_logger(filename, rank, name='parsl', level=logging.DEBUG, format_
        -  None
     """
     if format_string is None:
-        format_string = "%(asctime)s.%(msecs)03d %(name)s:%(lineno)d Rank:{0} [%(levelname)s]  %(message)s".format(rank)
+        format_string = "%(asctime)s.%(msecs)03d %(name)s:%(lineno)d " \
+                        "%(process)d %(threadName)s " \
+                        "[%(levelname)s]  %(message)s"
 
-    global logger
     logger = logging.getLogger(name)
     logger.setLevel(logging.DEBUG)
     handler = logging.FileHandler(filename)
@@ -559,6 +614,7 @@ def start_file_logger(filename, rank, name='parsl', level=logging.DEBUG, format_
     formatter = logging.Formatter(format_string, datefmt='%Y-%m-%d %H:%M:%S')
     handler.setFormatter(formatter)
     logger.addHandler(handler)
+    return logger
 
 
 if __name__ == "__main__":
@@ -594,15 +650,19 @@ if __name__ == "__main__":
                         help="Poll period used in milliseconds")
     parser.add_argument("-r", "--result_port", required=True,
                         help="REQUIRED: Result port for posting results to the interchange")
+    parser.add_argument("--cpu-affinity", type=str, choices=["none", "block", "alternating"],
+                        help="Whether/how workers should control CPU affinity.")
+    parser.add_argument("--available-accelerators", type=str, nargs="*",
+                        help="Names of available accelerators")
 
     args = parser.parse_args()
 
     os.makedirs(os.path.join(args.logdir, "block-{}".format(args.block_id), args.uid), exist_ok=True)
 
     try:
-        start_file_logger('{}/block-{}/{}/manager.log'.format(args.logdir, args.block_id, args.uid),
-                          0,
-                          level=logging.DEBUG if args.debug is True else logging.INFO)
+        logger = start_file_logger('{}/block-{}/{}/manager.log'.format(args.logdir, args.block_id, args.uid),
+                                   0,
+                                   level=logging.DEBUG if args.debug is True else logging.INFO)
 
         logger.info("Python version: {}".format(sys.version))
         logger.info("Debug logging: {}".format(args.debug))
@@ -620,6 +680,8 @@ if __name__ == "__main__":
         logger.info("Prefetch capacity: {}".format(args.prefetch_capacity))
         logger.info("Heartbeat threshold: {}".format(args.hb_threshold))
         logger.info("Heartbeat period: {}".format(args.hb_period))
+        logger.info("CPU affinity: {}".format(args.cpu_affinity))
+        logger.info("Accelerators: {}".format(" ".join(args.available_accelerators)))
 
         manager = Manager(task_port=args.task_port,
                           result_port=args.result_port,
@@ -633,13 +695,14 @@ if __name__ == "__main__":
                           prefetch_capacity=int(args.prefetch_capacity),
                           heartbeat_threshold=int(args.hb_threshold),
                           heartbeat_period=int(args.hb_period),
-                          poll_period=int(args.poll))
+                          poll_period=int(args.poll),
+                          cpu_affinity=args.cpu_affinity,
+                          available_accelerators=args.available_accelerators)
         manager.start()
 
-    except Exception as e:
-        logger.critical("process_worker_pool exiting from an exception")
-        logger.exception("Caught error: {}".format(e))
+    except Exception:
+        logger.critical("Process worker pool exiting with an exception", exc_info=True)
         raise
     else:
-        logger.info("process_worker_pool exiting")
-        print("PROCESS_WORKER_POOL exiting")
+        logger.info("Process worker pool exiting normally")
+        print("Process worker pool exiting normally")
