@@ -66,6 +66,8 @@ class CommandClient(object):
                     self.context.destroy()
                     self.context = zmq.Context()
                     self.create_socket_and_bind()
+                else:
+                    break
 
         if reply == '__PARSL_ZMQ_PIPES_MAGIC__':
             logger.error("Command channel run retries exhausted. Unable to run command")
@@ -110,7 +112,7 @@ class TasksOutgoing(object):
         in ZMQ sockets reaching a broken state once there are ~10k tasks in flight.
         This issue can be magnified if each the serialized buffer itself is larger.
         """
-        timeout_ms = 0
+        timeout_ms = 1
         while True:
             socks = dict(self.poller.poll(timeout=timeout_ms))
             if self.zmq_socket in socks and socks[self.zmq_socket] == zmq.POLLOUT:
@@ -118,8 +120,8 @@ class TasksOutgoing(object):
                 self.zmq_socket.send_pyobj(message, copy=True)
                 return
             else:
-                timeout_ms += 1
-                logger.debug("Not sending due to full zmq pipe, timeout: {} ms".format(timeout_ms))
+                timeout_ms *= 2
+                logger.debug("Not sending due to non-ready zmq pipe, timeout: {} ms".format(timeout_ms))
 
     def close(self):
         self.zmq_socket.close()
