@@ -1,4 +1,4 @@
-"""WorkQueueExecutor utilizes the Work Queue distributed framework developed by the
+""" WorkQueueExecutor utilizes the Work Queue distributed framework developed by the
 Cooperative Computing Lab (CCL) at Notre Dame to provide a fault-tolerant,
 high-throughput system for delegating Parsl tasks to thousands of remote machines
 """
@@ -19,20 +19,21 @@ import queue
 import inspect
 import shutil
 import itertools
-from ipyparallel.serialize import pack_apply_message
 
+from parsl.serialize import pack_apply_message
 import parsl.utils as putils
 from parsl.executors.errors import ExecutorError
 from parsl.data_provider.files import File
-from parsl.executors.status_handling import NoStatusHandlingExecutor
+from parsl.errors import OptionalModuleMissing
+from parsl.executors.status_handling import BlockProviderExecutor
 from parsl.providers.provider_base import ExecutionProvider
 from parsl.providers import LocalProvider, CondorProvider
-from parsl.providers.error import OptionalModuleMissing
-from parsl.executors.errors import ScalingFailed
 from parsl.executors.workqueue import exec_parsl_function
+from parsl.process_loggers import wrap_with_logs
+from parsl.utils import setproctitle
 
 import typeguard
-from typing import Dict, List, Optional, Set
+from typing import Dict, List, Optional, Set, Union
 from parsl.data_provider.staging import Staging
 
 from .errors import WorkQueueTaskFailure
@@ -60,7 +61,8 @@ logger = logging.getLogger(__name__)
 
 
 # Support structure to communicate parsl tasks to the work queue submit thread.
-ParslTaskToWq = namedtuple('ParslTaskToWq', 'id category env_pkg map_file function_file result_file input_files output_files')
+ParslTaskToWq = namedtuple('ParslTaskToWq',
+                           'id category cores memory disk gpus priority running_time_min env_pkg map_file function_file result_file input_files output_files')
 
 # Support structure to communicate final status of work queue tasks to parsl
 # result is only valid if result_received is True
@@ -74,7 +76,7 @@ WqTaskToParsl = namedtuple('WqTaskToParsl', 'id result_received result reason st
 ParslFileToWq = namedtuple('ParslFileToWq', 'parsl_name stage cache')
 
 
-class WorkQueueExecutor(NoStatusHandlingExecutor):
+class WorkQueueExecutor(BlockProviderExecutor, putils.RepresentationMixin):
     """Executor to use Work Queue batch system
 
     The WorkQueueExecutor system utilizes the Work Queue framework to
@@ -85,8 +87,8 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
     machines for execution and retrieval.
 
 
-        Parameters
-        ----------
+    Parameters
+    ----------
 
         label: str
             A human readable label for the executor, unique
@@ -102,8 +104,10 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
             Default is True (managed by DFK).
 
         project_name: str
-            If given, Work Queue master process name. Default is None.
-            Overrides address.
+            If a project_name is given, then Work Queue will periodically
+            report its status and performance back to the global WQ catalog,
+            which can be viewed here:  http://ccl.cse.nd.edu/software/workqueue/status
+            Default is None.  Overrides address.
 
         project_password_file: str
             Optional password file for the work queue project. Default is None.
@@ -149,6 +153,13 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
             require a common environment or shared FS on execution nodes.
             Implies source=True.
 
+        extra_pkgs: list
+            List of extra pip/conda package names to include when packing
+            the environment. This may be useful if the app executes other
+            (possibly non-Python) programs provided via pip or conda.
+            Scanning the app source for imports would not detect these
+            dependencies, so they need to be manually specified.
+
         autolabel: bool
             Use the Resource Monitor to automatically determine resource
             labels based on observed task behavior.
@@ -166,11 +177,29 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
             invocations of an app have similar performance characteristics,
             this will provide a reasonable set of categories automatically.
 
+        max_retries: Optional[int]
+            Set the number of retries that Work Queue will make when a task
+            fails. This is distinct from Parsl level retries configured in
+            parsl.config.Config. Set to None to allow Work Queue to retry
+            tasks forever. By default, this is set to 1, so that all retries
+            will be managed by Parsl.
+
         init_command: str
             Command line to run before executing a task in a worker.
             Default is ''.
+
+        worker_options: str
+            Extra options passed to work_queue_worker. Default is ''.
+
+        worker_executable: str
+            The command used to invoke work_queue_worker. This can be used
+            when the worker needs to be wrapped inside some other command
+            (for example, to run the worker inside a container). Default is
+            'work_queue_worker'.
     """
 
+    radio_mode = "filesystem"
+
     @typeguard.typechecked
     def __init__(self,
                  label: str = "WorkQueueExecutor",
@@ -187,13 +216,17 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
                  use_cache: bool = False,
                  source: bool = False,
                  pack: bool = False,
+                 extra_pkgs: Optional[List[str]] = None,
                  autolabel: bool = False,
                  autolabel_window: int = 1,
-                 autocategory: bool = False,
+                 autocategory: bool = True,
+                 max_retries: Optional[int] = 1,
                  init_command: str = "",
-                 full_debug: bool = True):
-        NoStatusHandlingExecutor.__init__(self)
-        self._provider = provider
+                 worker_options: str = "",
+                 full_debug: bool = True,
+                 worker_executable: str = 'work_queue_worker'):
+        BlockProviderExecutor.__init__(self, provider=provider,
+                                       block_error_handler=True)
         self._scaling_enabled = True
 
         if not _work_queue_enabled:
@@ -216,14 +249,18 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
         self.use_cache = use_cache
         self.working_dir = working_dir
         self.registered_files = set()  # type: Set[str]
-        self.full = full_debug
+        self.full_debug = full_debug
         self.source = True if pack else source
         self.pack = pack
+        self.extra_pkgs = extra_pkgs or []
         self.autolabel = autolabel
         self.autolabel_window = autolabel_window
         self.autocategory = autocategory
+        self.max_retries = max_retries
         self.should_stop = multiprocessing.Value(c_bool, False)
         self.cached_envs = {}  # type: Dict[int, str]
+        self.worker_options = worker_options
+        self.worker_executable = worker_executable
 
         if not self.address:
             self.address = socket.gethostname()
@@ -241,6 +278,11 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
         if self.init_command != "":
             self.launch_cmd = self.init_command + "; " + self.launch_cmd
 
+    def _get_launch_command(self, block_id):
+        # this executor uses different terminology for worker/launch
+        # commands than in htex
+        return f"PARSL_WORKER_BLOCK_ID={block_id} {self.worker_command}"
+
     def start(self):
         """Create submit process and collector thread to create, send, and
         retrieve Parsl tasks within the Work Queue system.
@@ -256,29 +298,30 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
         os.mkdir(self.package_dir)
         os.mkdir(self.wq_log_dir)
 
-        logger.debug("Starting WorkQueueExectutor")
+        logger.debug("Starting WorkQueueExecutor")
 
         # Create a Process to perform WorkQueue submissions
         submit_process_kwargs = {"task_queue": self.task_queue,
                                  "launch_cmd": self.launch_cmd,
                                  "data_dir": self.function_data_dir,
                                  "collector_queue": self.collector_queue,
-                                 "full": self.full,
+                                 "full": self.full_debug,
                                  "shared_fs": self.shared_fs,
                                  "autolabel": self.autolabel,
                                  "autolabel_window": self.autolabel_window,
                                  "autocategory": self.autocategory,
+                                 "max_retries": self.max_retries,
                                  "should_stop": self.should_stop,
                                  "port": self.port,
                                  "wq_log_dir": self.wq_log_dir,
                                  "project_password_file": self.project_password_file,
                                  "project_name": self.project_name}
         self.submit_process = multiprocessing.Process(target=_work_queue_submit_wait,
-                                                      name="submit_thread",
+                                                      name="WorkQueue-Submit-Process",
                                                       kwargs=submit_process_kwargs)
 
         self.collector_thread = threading.Thread(target=self._collect_work_queue_results,
-                                                 name="wait_thread")
+                                                 name="WorkQueue-collector-thread")
         self.collector_thread.daemon = True
 
         # Begin both processes
@@ -316,6 +359,55 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
         kwargs : dict
             Keyword arguments to the Parsl app
         """
+        cores = None
+        memory = None
+        disk = None
+        gpus = None
+        priority = None
+        category = None
+        running_time_min = None
+        if resource_specification and isinstance(resource_specification, dict):
+            logger.debug("Got resource specification: {}".format(resource_specification))
+
+            required_resource_types = set(['cores', 'memory', 'disk'])
+            acceptable_resource_types = set(['cores', 'memory', 'disk', 'gpus', 'priority', 'running_time_min'])
+            keys = set(resource_specification.keys())
+
+            if not keys.issubset(acceptable_resource_types):
+                message = "Task resource specification only accepts these types of resources: {}".format(
+                        ', '.join(acceptable_resource_types))
+                logger.error(message)
+                raise ExecutorError(self, message)
+
+            # this checks that either all of the required resource types are specified, or
+            # that none of them are: the `required_resource_types` are not actually required,
+            # but if one is specified, then they all must be.
+            key_check = required_resource_types.intersection(keys)
+            required_keys_ok = len(key_check) == 0 or key_check == required_resource_types
+            if not self.autolabel and not required_keys_ok:
+                logger.error("Running with `autolabel=False`. In this mode, "
+                             "task resource specification requires "
+                             "three resources to be specified simultaneously: cores, memory, and disk")
+                raise ExecutorError(self, "Task resource specification requires "
+                                          "three resources to be specified simultaneously: cores, memory, and disk. "
+                                          "Try setting autolabel=True if you are unsure of the resource usage")
+
+            for k in keys:
+                if k == 'cores':
+                    cores = resource_specification[k]
+                elif k == 'memory':
+                    memory = resource_specification[k]
+                elif k == 'disk':
+                    disk = resource_specification[k]
+                elif k == 'gpus':
+                    gpus = resource_specification[k]
+                elif k == 'priority':
+                    priority = resource_specification[k]
+                elif k == 'category':
+                    category = resource_specification[k]
+                elif k == 'running_time_min':
+                    running_time_min = resource_specification[k]
+
         self.task_counter += 1
         task_id = self.task_counter
 
@@ -343,7 +435,10 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
 
         # Create a Future object and have it be mapped from the task ID in the tasks dictionary
         fu = Future()
+        fu.parsl_executor_task_id = task_id
+        logger.debug("Getting tasks_lock to set WQ-level task entry")
         with self.tasks_lock:
+            logger.debug("Got tasks_lock to set WQ-level task entry")
             self.tasks[str(task_id)] = fu
 
         logger.debug("Creating task {} for function {} with args {}".format(task_id, func, args))
@@ -359,7 +454,7 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
         self._serialize_function(function_file, func, args, kwargs)
 
         if self.pack:
-            env_pkg = self._prepare_package(func)
+            env_pkg = self._prepare_package(func, self.extra_pkgs)
         else:
             env_pkg = None
 
@@ -371,15 +466,31 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
 
         # Create message to put into the message queue
         logger.debug("Placing task {} on message queue".format(task_id))
-        category = func.__qualname__ if self.autocategory else 'parsl-default'
-        self.task_queue.put_nowait(ParslTaskToWq(task_id, category, env_pkg, map_file, function_file, result_file, input_files, output_files))
+        if category is None:
+            category = func.__name__ if self.autocategory else 'parsl-default'
+        self.task_queue.put_nowait(ParslTaskToWq(task_id,
+                                                 category,
+                                                 cores,
+                                                 memory,
+                                                 disk,
+                                                 gpus,
+                                                 priority,
+                                                 running_time_min,
+                                                 env_pkg,
+                                                 map_file,
+                                                 function_file,
+                                                 result_file,
+                                                 input_files,
+                                                 output_files))
 
         return fu
 
     def _construct_worker_command(self):
-        worker_command = 'work_queue_worker'
+        worker_command = self.worker_executable
         if self.project_password_file:
             worker_command += ' --password {}'.format(self.project_password_file)
+        if self.worker_options:
+            worker_command += ' {}'.format(self.worker_options)
         if self.project_name:
             worker_command += ' -M {}'.format(self.project_name)
         else:
@@ -411,8 +522,8 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
                              "kwargs": parsl_fn_kwargs}
         else:
             function_info = {"byte code": pack_apply_message(parsl_fn, parsl_fn_args, parsl_fn_kwargs,
-                                                             buffer_threshold=1024 * 1024,
-                                                             item_threshold=1024)}
+                                                             buffer_threshold=1024 * 1024)}
+
         with open(fn_path, "wb") as f_out:
             pickle.dump(function_info, f_out)
 
@@ -464,9 +575,9 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
         to_stage = not os.path.isabs(fname)
         return ParslFileToWq(fname, stage=to_stage, cache=False)
 
-    def _prepare_package(self, fn):
+    def _prepare_package(self, fn, extra_pkgs):
         fn_id = id(fn)
-        fn_name = fn.__qualname__
+        fn_name = fn.__name__
         if fn_id in self.cached_envs:
             logger.debug("Skipping analysis of %s, previously got %s", fn_name, self.cached_envs[fn_id])
             return self.cached_envs[fn_id]
@@ -475,7 +586,10 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
         os.makedirs(pkg_dir, exist_ok=True)
         with tempfile.NamedTemporaryFile(suffix='.yaml') as spec:
             logger.info("Analyzing dependencies of %s", fn_name)
-            subprocess.run([package_analyze_script, '-', spec.name], input=source_code, check=True)
+            analyze_cmdline = [package_analyze_script, exec_parsl_function.__file__, '-', spec.name]
+            for p in extra_pkgs:
+                analyze_cmdline += ["--extra-pkg", p]
+            subprocess.run(analyze_cmdline, input=source_code, check=True)
             with open(spec.name, mode='rb') as f:
                 spec_hash = hashlib.sha256(f.read()).hexdigest()
                 logger.debug("Spec hash for %s is %s", fn_name, spec_hash)
@@ -508,27 +622,25 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
             try:
                 self.scale_out(blocks=self.provider.init_blocks)
             except Exception as e:
-                logger.debug("Scaling out failed: {}".format(e))
+                logger.error("Initial block scaling out failed: {}".format(e))
                 raise e
 
-    def scale_out(self, blocks=1):
-        """Scale out method.
-
-        We should have the scale out method simply take resource object
-        which will have the scaling methods, scale_out itself should be a coroutine, since
-        scaling tasks can be slow.
+    @property
+    def outstanding(self) -> int:
+        """Count the number of outstanding tasks. This is inefficiently
+        implemented and probably could be replaced with a counter.
         """
-        if self.provider:
-            for i in range(blocks):
-                external_block = str(len(self.blocks))
-                internal_block = self.provider.submit(self.worker_command, 1)
-                # Failed to create block with provider
-                if not internal_block:
-                    raise(ScalingFailed(self.provider.label, "Attempts to create nodes using the provider has failed"))
-                else:
-                    self.blocks[external_block] = internal_block
-        else:
-            logger.error("No execution provider available to scale")
+        outstanding = 0
+        with self.tasks_lock:
+            for fut in self.tasks.values():
+                if not fut.done():
+                    outstanding += 1
+        logger.debug(f"Counted {outstanding} outstanding tasks")
+        return outstanding
+
+    @property
+    def workers_per_node(self) -> Union[int, float]:
+        return 1
 
     def scale_in(self, count):
         """Scale in method. Not implemented.
@@ -547,16 +659,21 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
         """Shutdown the executor. Sets flag to cancel the submit process and
         collector thread, which shuts down the Work Queue system submission.
         """
+        logger.debug("Work Queue shutdown started")
         self.should_stop.value = True
 
         # Remove the workers that are still going
         kill_ids = [self.blocks[block] for block in self.blocks.keys()]
         if self.provider:
+            logger.debug("Cancelling blocks")
             self.provider.cancel(kill_ids)
 
+        logger.debug("Joining on submit process")
         self.submit_process.join()
+        logger.debug("Joining on collector thread")
         self.collector_thread.join()
 
+        logger.debug("Work Queue shutdown completed")
         return True
 
     def scaling_enabled(self):
@@ -571,6 +688,7 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
             self._run_dir = value
         return self._run_dir
 
+    @wrap_with_logs
     def _collect_work_queue_results(self):
         """Sets the values of tasks' futures of tasks completed by work queue.
         """
@@ -598,7 +716,10 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
                     # work queue modes, such as resource exhaustion.
                     future.set_exception(WorkQueueTaskFailure(task_report.reason, task_report.result))
         finally:
+            logger.debug("Marking all outstanding tasks as failed")
+            logger.debug("Acquiring tasks_lock")
             with self.tasks_lock:
+                logger.debug("Acquired tasks_lock")
                 # set exception for tasks waiting for results that work queue did not execute
                 for fu in self.tasks.values():
                     if not fu.done():
@@ -606,6 +727,7 @@ class WorkQueueExecutor(NoStatusHandlingExecutor):
         logger.debug("Exiting Collector Thread")
 
 
+@wrap_with_logs
 def _work_queue_submit_wait(task_queue=multiprocessing.Queue(),
                             launch_cmd=None,
                             env=None,
@@ -616,6 +738,7 @@ def _work_queue_submit_wait(task_queue=multiprocessing.Queue(),
                             autolabel=False,
                             autolabel_window=None,
                             autocategory=False,
+                            max_retries=0,
                             should_stop=None,
                             port=WORK_QUEUE_DEFAULT_PORT,
                             wq_log_dir=None,
@@ -633,6 +756,7 @@ def _work_queue_submit_wait(task_queue=multiprocessing.Queue(),
     module capabilities, rather than shared memory.
     """
     logger.debug("Starting WorkQueue Submit/Wait Process")
+    setproctitle("parsl: Work Queue submit/wait")
 
     # Enable debugging flags and create logging file
     wq_debug_log = None
@@ -665,7 +789,7 @@ def _work_queue_submit_wait(task_queue=multiprocessing.Queue(),
     if wq_log_dir is not None:
         wq_master_log = os.path.join(wq_log_dir, "master_log")
         wq_trans_log = os.path.join(wq_log_dir, "transaction_log")
-        if full:
+        if full and autolabel:
             wq_resource_log = os.path.join(wq_log_dir, "resource_logs")
             q.enable_monitoring_full(dirname=wq_resource_log)
         q.specify_log(wq_master_log)
@@ -721,6 +845,25 @@ def _work_queue_submit_wait(task_queue=multiprocessing.Queue(),
             if autolabel:
                 q.specify_category_mode(task.category, WORK_QUEUE_ALLOCATION_MODE_MAX_THROUGHPUT)
 
+            if task.cores is not None:
+                t.specify_cores(task.cores)
+            if task.memory is not None:
+                t.specify_memory(task.memory)
+            if task.disk is not None:
+                t.specify_disk(task.disk)
+            if task.gpus is not None:
+                t.specify_gpus(task.gpus)
+            if task.priority is not None:
+                t.specify_priority(task.priority)
+            if task.running_time_min is not None:
+                t.specify_running_time_min(task.running_time_min)
+
+            if max_retries is not None:
+                logger.debug(f"Specifying max_retries {max_retries}")
+                t.specify_max_retries(max_retries)
+            else:
+                logger.debug("Not specifying max_retries")
+
             # Specify environment variables for the task
             if env is not None:
                 for var in env:
@@ -763,7 +906,7 @@ def _work_queue_submit_wait(task_queue=multiprocessing.Queue(),
                                                          reason="task could not be submited to work queue",
                                                          status=-1))
                 continue
-            logger.debug("Task {} submitted to WorkQueue with id {}".format(task.id, wq_id))
+            logger.info("Task {} submitted to WorkQueue with id {}".format(task.id, wq_id))
 
         # If the queue is not empty wait on the WorkQueue queue for a task
         task_found = True
