@@ -4,7 +4,7 @@ from parsl.providers.kubernetes.template import template_string
 
 logger = logging.getLogger(__name__)
 
-from parsl.providers.error import OptionalModuleMissing
+from parsl.errors import OptionalModuleMissing
 from parsl.providers.provider_base import ExecutionProvider, JobState, JobStatus
 from parsl.utils import RepresentationMixin
 
@@ -17,11 +17,21 @@ try:
 except (ImportError, NameError, FileNotFoundError):
     _kubernetes_enabled = False
 
+translate_table = {
+    'Running': JobState.RUNNING,
+    'Pending': JobState.PENDING,
+    'Succeeded': JobState.COMPLETED,
+    'Failed': JobState.FAILED,
+    'Unknown': JobState.UNKNOWN,
+}
+
 
 class KubernetesProvider(ExecutionProvider, RepresentationMixin):
     """ Kubernetes execution provider
+
     Parameters
     ----------
+
     namespace : str
         Kubernetes namespace to create deployments.
     image : str
@@ -55,9 +65,9 @@ class KubernetesProvider(ExecutionProvider, RepresentationMixin):
         scaling where as many resources as possible are used; parallelism close to 0 represents
         the opposite situation in which as few resources as possible (i.e., min_blocks) are used.
     worker_init : str
-        Command to be run first for the workers, such as `python start.py`.
+        Command to be run first for the workers, such as ``python start.py``.
     secret : str
-        Docker secret to use to pull images
+        The Kubernetes ImagePullSecret secret to use to pull images
     pod_name : str
         The name for the pod, will be appended with a timestamp.
         Default is None, meaning parsl automatically names the pod.
@@ -118,7 +128,7 @@ class KubernetesProvider(ExecutionProvider, RepresentationMixin):
         self.kube_client = client.CoreV1Api()
 
         # Dictionary that keeps track of jobs, keyed on job_id
-        self.resources = {}  # type: Dict[str, Dict[str, Any]]
+        self.resources = {}  # type: Dict[object, Dict[str, Any]]
 
     def submit(self, cmd_string, tasks_per_node, job_name="parsl"):
         """ Submit a job
@@ -128,6 +138,7 @@ class KubernetesProvider(ExecutionProvider, RepresentationMixin):
 
         Kwargs:
              - job_name (String): Name for job, must be unique
+
         Returns:
              - None: At capacity, cannot provision more
              - job_id: (string) Identifier for the job
@@ -165,9 +176,9 @@ class KubernetesProvider(ExecutionProvider, RepresentationMixin):
         Raises:
              - ExecutionProviderExceptions or its subclasses
         """
-        self._status()
-        # This is a hack
-        return [JobStatus(JobState.RUNNING) for jid in job_ids]
+        if job_ids:
+            self._status()
+        return [self.resources[jid]['status'] for jid in job_ids]
 
     def cancel(self, job_ids):
         """ Cancels the jobs specified by a list of job ids
@@ -177,28 +188,41 @@ class KubernetesProvider(ExecutionProvider, RepresentationMixin):
         [True/False...] : If the cancel operation fails the entire list will be False.
         """
         for job in job_ids:
-            logger.debug("Terminating job/proc_id: {0}".format(job))
-            # Here we are assuming that for local, the job_ids are the process id's
+            logger.debug("Terminating job/pod: {0}".format(job))
             self._delete_pod(job)
 
             self.resources[job]['status'] = JobStatus(JobState.CANCELLED)
-            del self.resources[job]
         rets = [True for i in job_ids]
 
         return rets
 
     def _status(self):
-        """ Internal: Do not call. Returns the status list for a list of job_ids
+        """Returns the status list for a list of job_ids
         Args:
               self
         Returns:
               [status...] : Status list of all jobs
         """
 
-        jobs_ids = list(self.resources.keys())
-        # TODO: fix this
-        return jobs_ids
-        # do something to get the deployment's status
+        job_ids = list(self.resources.keys())
+        to_poll_job_ids = [jid for jid in job_ids if not self.resources[jid]['status'].terminal]
+        logger.debug("Polling Kubernetes pod status: {}".format(to_poll_job_ids))
+        for jid in to_poll_job_ids:
+            phase = None
+            try:
+                pod_status = self.kube_client.read_namespaced_pod_status(name=jid, namespace=self.namespace)
+            except Exception:
+                logger.exception("Failed to poll pod {} status, most likely because pod was terminated".format(jid))
+                if self.resources[jid]['status'] is JobStatus(JobState.RUNNING):
+                    phase = 'Unknown'
+            else:
+                phase = pod_status.status.phase
+            if phase:
+                status = translate_table.get(phase, JobState.UNKNOWN)
+                logger.debug("Updating pod {} with status {} to parsl status {}".format(jid,
+                                                                                        phase,
+                                                                                        status))
+                self.resources[jid]['status'] = JobStatus(status)
 
     def _create_pod(self,
                     image,
