@@ -1,6 +1,7 @@
 import logging
 import os
 import time
+import json
 
 from parsl.channels import LocalChannel
 from parsl.launchers import SingleNodeLauncher
@@ -8,16 +9,14 @@ from parsl.providers.pbspro.template import template_string
 from parsl.providers import TorqueProvider
 from parsl.providers.provider_base import JobState, JobStatus
 
+from parsl.providers.torque.torque import translate_table
+
 logger = logging.getLogger(__name__)
 
 
 class PBSProProvider(TorqueProvider):
     """PBS Pro Execution Provider
 
-    This provider uses sbatch to submit, squeue for status, and scancel to cancel
-    jobs. The sbatch script to be used is created from a template file in this
-    same module.
-
     Parameters
     ----------
     channel : Channel
@@ -47,6 +46,9 @@ class PBSProProvider(TorqueProvider):
         Walltime requested per block in HH:MM:SS.
     scheduler_options : str
         String to prepend to the #PBS blocks in the submit script to the scheduler.
+    select_options : str
+        String to append to the #PBS -l select block in the submit script to the scheduler. This can be used to
+        specify ngpus.
     worker_init : str
         Command to be run before starting a worker, such as 'module load Anaconda; source activate env'.
     launcher : Launcher
@@ -58,6 +60,7 @@ class PBSProProvider(TorqueProvider):
                  account=None,
                  queue=None,
                  scheduler_options='',
+                 select_options='',
                  worker_init='',
                  nodes_per_block=1,
                  cpus_per_node=1,
@@ -85,6 +88,44 @@ class PBSProProvider(TorqueProvider):
         self.template_string = template_string
         self._label = 'pbspro'
         self.cpus_per_node = cpus_per_node
+        self.select_options = select_options
+
+    def _status(self):
+        ''' Internal: Do not call. Returns the status list for a list of job_ids
+
+        Args:
+              self
+
+        Returns:
+              [status...] : Status list of all jobs
+        '''
+
+        job_ids = list(self.resources.keys())
+        job_id_list = ' '.join(self.resources.keys())
+
+        jobs_missing = list(self.resources.keys())
+
+        retcode, stdout, stderr = self.execute_wait("qstat -f -F json {0}".format(job_id_list))
+
+        job_statuses = json.loads(stdout)
+
+        if 'Jobs' in job_statuses:
+            for job_id, job in job_statuses['Jobs'].items():
+                for long_job_id in job_ids:
+                    if long_job_id.startswith(job_id):
+                        logger.debug('coerced job_id %s -> %s', job_id, long_job_id)
+                        job_id = long_job_id
+                        break
+
+                job_state = job.get('job_state', JobState.UNKNOWN)
+                state = translate_table.get(job_state, JobState.UNKNOWN)
+                self.resources[job_id]['status'] = JobStatus(state)
+                jobs_missing.remove(job_id)
+
+        # squeue does not report on jobs that are not running. So we are filling in the
+        # blanks for missing jobs, we might lose some information about why the jobs failed.
+        for missing_job in jobs_missing:
+            self.resources[missing_job]['status'] = JobStatus(JobState.COMPLETED)
 
     def submit(self, command, tasks_per_node, job_name="parsl"):
         """Submits the command job.
@@ -106,10 +147,6 @@ class PBSProProvider(TorqueProvider):
             Identifier for the job
         """
 
-        if self.provisioned_blocks >= self.max_blocks:
-            logger.warning("[%s] at capacity, cannot add more blocks now", self.label)
-            return None
-
         job_name = "{0}.{1}".format(job_name, time.time())
 
         script_path = os.path.abspath("{0}/{1}.submit".format(self.script_dir, job_name))
@@ -127,6 +164,12 @@ class PBSProProvider(TorqueProvider):
         job_config["worker_init"] = self.worker_init
         job_config["user_script"] = command
 
+        # Add a colon to select_options if one isn't included
+        if self.select_options and not self.select_options.startswith(":"):
+            self.select_options = ":" + self.select_options
+
+        job_config["select_options"] = self.select_options
+
         # Wrap the command
         job_config["user_script"] = self.launcher(command,
                                                   tasks_per_node,
