@@ -20,15 +20,15 @@ translate_table = {
     'H': JobState.HELD,  # Held
     'Q': JobState.PENDING,  # Queued, and eligible to run
     'W': JobState.PENDING,  # Job is waiting for it's execution time (-a option) to be reached
-    'S': JobState.HELD
-}  # Suspended
+    'S': JobState.HELD  # Suspended
+}
 
 
 class TorqueProvider(ClusterProvider, RepresentationMixin):
     """Torque Execution Provider
 
-    This provider uses sbatch to submit, squeue for status, and scancel to cancel
-    jobs. The sbatch script to be used is created from a template file in this
+    This provider uses qsub to submit, qstat for status, and qdel to cancel
+    jobs. The qsub script to be used is created from a template file in this
     same module.
 
     Parameters
@@ -97,7 +97,6 @@ class TorqueProvider(ClusterProvider, RepresentationMixin):
         self.queue = queue
         self.scheduler_options = scheduler_options
         self.worker_init = worker_init
-        self.provisioned_blocks = 0
         self.template_string = template_string
 
         # Dictionary that keeps track of jobs, keyed on job_id
@@ -163,10 +162,6 @@ class TorqueProvider(ClusterProvider, RepresentationMixin):
 
         '''
 
-        if self.provisioned_blocks >= self.max_blocks:
-            logger.warning("[%s] at capacity, cannot add more blocks now", self.label)
-            return None
-
         # Set job name
         job_name = "parsl.{0}.{1}".format(job_name, time.time())
 
@@ -247,8 +242,3 @@ class TorqueProvider(ClusterProvider, RepresentationMixin):
     @property
     def status_polling_interval(self):
         return 60
-
-
-if __name__ == "__main__":
-
-    print("None")
