@@ -1,7 +1,6 @@
 from parsl.config import Config
 from parsl.providers import SlurmProvider
 from parsl.launchers import SrunLauncher
-from parsl.addresses import address_by_hostname
 from parsl.executors import HighThroughputExecutor
 
 from .user_opts import user_opts
@@ -13,7 +12,6 @@ def fresh_config():
             HighThroughputExecutor(
                 label='Midway_HTEX_multinode',
                 worker_debug=False,
-                address=address_by_hostname(),
                 max_workers=1,
                 provider=SlurmProvider(
                     'broadwl',  # Partition name, e.g 'broadwl'
