@@ -17,6 +17,11 @@ def fresh_config():
         from parsl.tests.configs.frontera import fresh_config
         config = fresh_config()
 
+    elif 'nscc' in hostname:
+        print("Loading NSCC Singapore config")
+        from parsl.tests.configs.nscc_singapore import fresh_config
+        config = fresh_config()
+
     elif 'summit' in socket.getfqdn():
         print("Loading Frontera config")
         from parsl.tests.configs.summit import fresh_config
@@ -52,22 +57,17 @@ def fresh_config():
         from parsl.tests.configs.cc_in2p3 import fresh_config
         config = fresh_config()
 
+    elif 'bridges' in socket.getfqdn():
+        print("Loading bridges config")
+        from parsl.tests.configs.bridges import fresh_config
+        config = fresh_config()
+
     elif 'cooley' in hostname:
         print("on Cooley, loading PetrelKube config")
         from parsl.tests.configs.petrelkube import fresh_config
         config = fresh_config()
 
     else:
-        print("Loading Local HTEX config")
-        from parsl.tests.configs.htex_local import fresh_config
-        config = fresh_config()
-        config.executors[0].max_workers = 1
-        config.executors[0].provider.init_blocks = 1
-        config.executors[0].provider.min_blocks = 0
-        config.executors[0].provider.max_blocks = 1
-        # We should skip this.
+        raise RuntimeError("This site cannot be identified")
 
     return config
-
-
-config = fresh_config()
