@@ -8,6 +8,8 @@ from parsl.app.app import bash_app
 import parsl.app.errors as pe
 
 
+from parsl.app.errors import BashExitFailure
+
 from parsl.tests.configs.local_threads import config
 
 
@@ -32,12 +34,6 @@ def div_0(stderr='std.err', stdout='std.out'):
     return cmd_line
 
 
-@bash_app
-def invalid_exit(stderr='std.err', stdout='std.out'):
-    cmd_line = 'exit 3.141'
-    return cmd_line
-
-
 @bash_app
 def not_executable(stderr='std.err', stdout='std.out'):
     cmd_line = '/dev/null'
@@ -60,9 +56,6 @@ test_matrix = {
     command_not_found: {
         'exit_code': 127
     },
-    invalid_exit: {
-        'exit_code': 128
-    },
     not_executable: {
         'exit_code': 126
     }
@@ -71,28 +64,12 @@ test_matrix = {
 whitelist = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'configs', '*threads*')
 
 
-# @pytest.mark.whitelist(whitelist, reason='broken in IPP')
-@pytest.mark.skip("Broke somewhere between PR #525 and PR #652")
-def test_bash_formatting():
-
-    f = bad_format()
-    try:
-        f.result()
-    except Exception as e:
-        print("Caught exception", e)
-        assert isinstance(
-            e, parsl.app.errors.AppBadFormatting), "Expected AppBadFormatting got : {0}".format(e)
-    return True
-
-
-# @pytest.mark.whitelist(whitelist, reason='broken in IPP')
-@pytest.mark.skip("Broke somewhere between PR #525 and PR #652")
 def test_div_0(test_fn=div_0):
     err_code = test_matrix[test_fn]['exit_code']
     f = test_fn()
     try:
         f.result()
-    except Exception as e:
+    except BashExitFailure as e:
         print("Caught exception", e)
         assert e.exitcode == err_code, "{0} expected err_code:{1} but got {2}".format(test_fn.__name__,
                                                                                       err_code,
@@ -135,30 +112,13 @@ def test_command_not_found(test_fn=command_not_found):
     return True
 
 
-@pytest.mark.skip('broken')
-def test_invalid_exit(test_fn=invalid_exit):
-    err_code = test_matrix[test_fn]['exit_code']
-    f = test_fn()
-    try:
-        f.result()
-    except Exception as e:
-        print("Caught exception", e)
-        assert e.exitcode == err_code, "{0} expected err_code:{1} but got {2}".format(test_fn.__name__,
-                                                                                      err_code,
-                                                                                      e.exitcode)
-    os.remove('std.err')
-    os.remove('std.out')
-    return True
-
-
-# @pytest.mark.whitelist(whitelist, reason='broken in IPP')
-@pytest.mark.skip("Broke somewhere between PR #525 and PR #652")
+@pytest.mark.issue363
 def test_not_executable(test_fn=not_executable):
     err_code = test_matrix[test_fn]['exit_code']
     f = test_fn()
     try:
         f.result()
-    except Exception as e:
+    except BashExitFailure as e:
         print("Caught exception", e)
         assert e.exitcode == err_code, "{0} expected err_code:{1} but got {2}".format(test_fn.__name__,
                                                                                       err_code,
@@ -173,7 +133,7 @@ def run_app(test_fn, err_code):
     print(f)
     try:
         f.result()
-    except Exception as e:
+    except BashExitFailure as e:
         print("Caught exception", e)
         assert e.exitcode == err_code, "{0} expected err_code:{1} but got {2}".format(test_fn.__name__,
                                                                                       err_code,
@@ -190,8 +150,6 @@ if __name__ == '__main__':
                         help="Count of apps to launch")
     args = parser.parse_args()
 
-    print(test_bash_formatting())
-
     exit(0)
 
     if args.debug:
