@@ -47,7 +47,6 @@ def sleep(sleep_dur=0.1):
 
 
 @pytest.mark.local
-@pytest.mark.skip('passes even without retries enabled')
 def test_fail_nowait(numtasks=10):
     """Test retries on tasks with no dependencies.
     """
@@ -56,6 +55,9 @@ def test_fail_nowait(numtasks=10):
         fu = sleep_then_fail(sleep_dur=0.1)
         fus.extend([fu])
 
+    # wait for all tasks to complete before ending this test
+    [x.exception() for x in fus]
+
     try:
         [x.result() for x in fus]
     except Exception as e:
@@ -66,7 +68,6 @@ def test_fail_nowait(numtasks=10):
 
 
 @pytest.mark.local
-@pytest.mark.skip('passes even without retries enabled')
 def test_fail_delayed(numtasks=10):
     """Test retries on tasks with dependencies.
 
@@ -80,6 +81,9 @@ def test_fail_delayed(numtasks=10):
         fu = sleep_then_fail(inputs=[x], sleep_dur=0.5)
         fus.extend([fu])
 
+    # wait for all tasks to complete before ending this test
+    [x.exception() for x in fus]
+
     try:
         [x.result() for x in fus]
     except Exception as e:
