@@ -1,6 +1,5 @@
 import pytest
 
-import threading
 from parsl import python_app
 from parsl.tests.configs.htex_local import fresh_config
 from parsl.executors.errors import SerializationError
@@ -11,14 +10,21 @@ local_config.retries = 2
 
 @python_app
 def fail_pickling(x):
-    return True
+    return next(x)
+
+
+def generator():
+    num = 0
+    while True:
+        yield num
+        num += 1
 
 
 @pytest.mark.local
 def test_serialization_error():
 
-    lock = threading.Lock()
-    x = fail_pickling(lock)
+    gen = generator()
+    x = fail_pickling(gen)
 
     try:
         x.result()
